@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1997-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1997-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -205,7 +205,13 @@ static void
 erts_cleanup_message(ErtsMessage *mp)
 {
     ErlHeapFragment *bp;
-    if (ERTS_SIG_IS_EXTERNAL_MSG(mp) || ERTS_SIG_IS_NON_MSG(mp)) {
+
+    if (ERTS_SIG_IS_NON_MSG(mp)) {
+        erts_proc_sig_cleanup_non_msg_signal(mp);
+        return;
+    }
+
+    if (ERTS_SIG_IS_EXTERNAL_MSG(mp)) {
         ErtsDistExternal *edep = erts_proc_sig_get_external(mp);
         if (edep) {
             erts_free_dist_ext_copy(edep);
@@ -216,18 +222,12 @@ erts_cleanup_message(ErtsMessage *mp)
         }
     }
 
-    if (ERTS_SIG_IS_MSG(mp) && mp->data.attached != ERTS_MSG_COMBINED_HFRAG) {
+    if (mp->data.attached != ERTS_MSG_COMBINED_HFRAG)
         bp = mp->data.heap_frag;
-    } else {
-        /* All non msg signals are combined HFRAG messages,
-           but we overwrite the mp->data field with the
-           nm_signal queue ptr so have to fix that here
-           before freeing it. */
-        mp->data.attached = ERTS_MSG_COMBINED_HFRAG;
+    else {
         bp = mp->hfrag.next;
         erts_cleanup_offheap(&mp->hfrag.off_heap);
     }
-
     if (bp)
         free_message_buffer(bp);
 }
@@ -334,9 +334,6 @@ erts_queue_dist_message(Process *rcvr,
     else {
 	LINK_MESSAGE(rcvr, mp);
 
-        if (rcvr_locks & ERTS_PROC_LOCK_MAIN)
-            erts_proc_sig_fetch(rcvr);
-
 	if (!(rcvr_locks & ERTS_PROC_LOCK_MSGQ))
 	    erts_proc_unlock(rcvr, ERTS_PROC_LOCK_MSGQ);
 
@@ -401,9 +398,6 @@ queue_messages(Process* receiver,
         erts_enqueue_signals(receiver, first, last, NULL, len, state);
     }
 
-    if (receiver_locks & ERTS_PROC_LOCK_MAIN)
-        erts_proc_sig_fetch(receiver);
-
     if (locked_msgq) {
 	erts_proc_unlock(receiver, ERTS_PROC_LOCK_MSGQ);
     }
@@ -455,6 +449,24 @@ erts_queue_message(Process* receiver, ErtsProcLocks receiver_locks,
     queue_messages(receiver, receiver_locks, mp, &mp->next, 1);
 }
 
+/**
+ *
+ * @brief Send one message from *NOT* a local process.
+ *
+ * But with a token!
+ */
+void
+erts_queue_message_token(Process* receiver, ErtsProcLocks receiver_locks,
+                         ErtsMessage* mp, Eterm msg, Eterm from, Eterm token)
+{
+    ASSERT(is_not_internal_pid(from));
+    ERL_MESSAGE_TERM(mp) = msg;
+    ERL_MESSAGE_FROM(mp) = from;
+    ERL_MESSAGE_TOKEN(mp) = token;
+    queue_messages(receiver, receiver_locks, mp, &mp->next, 1);
+}
+
+
 /**
  * @brief Send one message from a local process.
  *
@@ -641,12 +653,16 @@ erts_send_message(Process* sender,
     Eterm utag = NIL;
 #endif
     erts_aint32_t receiver_state;
+#ifdef ERTS_ENABLE_LOCK_CHECK
+    int have_receiver_main_lock = 0;
+#endif
 #ifdef SHCOPY_SEND
     erts_shcopy_t info;
 #else
     erts_literal_area_t litarea;
     INITIALIZE_LITERAL_PURGE_AREA(litarea);
 #endif
+    
 
 #ifdef USE_VM_PROBES
     *sender_name = *receiver_name = '\0';
@@ -674,7 +690,7 @@ erts_send_message(Process* sender,
          * Make sure we don't use the heap between those instances.
          */
         if (have_seqtrace(stoken)) {
-	    seq_trace_update_send(sender);
+	    seq_trace_update_serial(sender);
 	    seq_trace_output(stoken, message, SEQ_TRACE_SEND,
 			     receiver->common.id, sender);
 
@@ -704,6 +720,13 @@ erts_send_message(Process* sender,
                                             + seq_trace_size),
                                            &hp,
                                            &ohp);
+#ifdef ERTS_ENABLE_LOCK_CHECK
+        if ((*receiver_locks) & ERTS_PROC_LOCK_MAIN) {
+            have_receiver_main_lock = 1;
+            erts_proc_lc_require_lock(receiver, ERTS_PROC_LOCK_MAIN,
+                                      __FILE__, __LINE__);
+        }
+#endif
 
 #ifdef SHCOPY_SEND
 	if (is_not_immed(message))
@@ -741,6 +764,12 @@ erts_send_message(Process* sender,
 	if (receiver == sender && !(receiver_state & ERTS_PSFLG_OFF_HEAP_MSGQ)) {
 	    mp = erts_alloc_message(0, NULL);
 	    msize = 0;
+#ifdef ERTS_ENABLE_LOCK_CHECK
+            ASSERT((*receiver_locks) & ERTS_PROC_LOCK_MAIN);
+            have_receiver_main_lock = 1;
+            erts_proc_lc_require_lock(receiver, ERTS_PROC_LOCK_MAIN,
+                                      __FILE__, __LINE__);
+#endif
 	}
 	else {
 #ifdef SHCOPY_SEND
@@ -755,6 +784,13 @@ erts_send_message(Process* sender,
 					       msize,
 					       &hp,
 					       &ohp);
+#ifdef ERTS_ENABLE_LOCK_CHECK
+            if ((*receiver_locks) & ERTS_PROC_LOCK_MAIN) {
+                have_receiver_main_lock = 1;
+                erts_proc_lc_require_lock(receiver, ERTS_PROC_LOCK_MAIN,
+                                          __FILE__, __LINE__);
+            }
+#endif
 #ifdef SHCOPY_SEND
             if (is_not_immed(message))
                 message = copy_shared_perform(message, msize, &info, &hp, ohp);
@@ -777,6 +813,11 @@ erts_send_message(Process* sender,
 
     erts_queue_proc_message(sender, receiver, *receiver_locks, mp, message);
 
+#ifdef ERTS_ENABLE_LOCK_CHECK
+    if (have_receiver_main_lock)
+        erts_proc_lc_unrequire_lock(receiver, ERTS_PROC_LOCK_MAIN);
+#endif
+
     if (msize > ERTS_MSG_COPY_WORDS_PER_REDUCTION) {
         Uint reds = msize / ERTS_MSG_COPY_WORDS_PER_REDUCTION;
         if (reds > CONTEXT_REDS)
@@ -897,118 +938,6 @@ void erts_save_message_in_proc(Process *p, ErtsMessage *msgp)
     p->msg_frag = msgp;
 }
 
-Sint
-erts_move_messages_off_heap(Process *c_p)
-{
-    int reds = 1;
-    int i;
-    ErtsMessage *msgq[] = {c_p->sig_qs.first, c_p->sig_qs.cont};
-    /*
-     * Move all messages off heap. This *only* occurs when the
-     * process had off heap message disabled and just enabled
-     * it...
-     */
-
-    reds += erts_proc_sig_privqs_len(c_p) / 10;
-
-    ASSERT(erts_atomic32_read_nob(&c_p->state)
-	   & ERTS_PSFLG_OFF_HEAP_MSGQ);
-    ASSERT(c_p->flags & F_OFF_HEAP_MSGQ_CHNG);
-
-    for (i = 0; i < sizeof(msgq)/sizeof(msgq[0]); i++) {
-        ErtsMessage *mp;
-        for (mp = msgq[i]; mp; mp = mp->next) {
-            Uint msg_sz, token_sz;
-#ifdef USE_VM_PROBES
-            Uint utag_sz;
-#endif
-            Eterm *hp;
-            ErlHeapFragment *hfrag;
-
-            if (!ERTS_SIG_IS_INTERNAL_MSG(mp))
-                continue;
-
-            if (mp->data.attached)
-                continue;
-
-            if (is_immed(ERL_MESSAGE_TERM(mp))
-#ifdef USE_VM_PROBES
-                && is_immed(ERL_MESSAGE_DT_UTAG(mp))
-#endif
-                && is_not_immed(ERL_MESSAGE_TOKEN(mp)))
-                continue;
-
-            /*
-             * The message refers into the heap. Copy the message
-             * from the heap into a heap fragment and attach
-             * it to the message...
-             */
-            msg_sz = size_object(ERL_MESSAGE_TERM(mp));
-#ifdef USE_VM_PROBES
-            utag_sz = size_object(ERL_MESSAGE_DT_UTAG(mp));
-#endif
-            token_sz = size_object(ERL_MESSAGE_TOKEN(mp));
-
-            hfrag = new_message_buffer(msg_sz
-#ifdef USE_VM_PROBES
-                                       + utag_sz
-#endif
-                                       + token_sz);
-            hp = hfrag->mem;
-            if (is_not_immed(ERL_MESSAGE_TERM(mp)))
-                ERL_MESSAGE_TERM(mp) = copy_struct(ERL_MESSAGE_TERM(mp),
-                                                   msg_sz, &hp,
-                                                   &hfrag->off_heap);
-            if (is_not_immed(ERL_MESSAGE_TOKEN(mp)))
-                ERL_MESSAGE_TOKEN(mp) = copy_struct(ERL_MESSAGE_TOKEN(mp),
-                                                    token_sz, &hp,
-                                                    &hfrag->off_heap);
-#ifdef USE_VM_PROBES
-            if (is_not_immed(ERL_MESSAGE_DT_UTAG(mp)))
-                ERL_MESSAGE_DT_UTAG(mp) = copy_struct(ERL_MESSAGE_DT_UTAG(mp),
-                                                      utag_sz, &hp,
-                                                      &hfrag->off_heap);
-#endif
-            mp->data.heap_frag = hfrag;
-            reds += 1;
-        }
-    }
-
-    return reds;
-}
-
-Sint
-erts_complete_off_heap_message_queue_change(Process *c_p)
-{
-    int reds = 1;
-
-    ERTS_LC_ASSERT(ERTS_PROC_LOCK_MAIN == erts_proc_lc_my_proc_locks(c_p));
-    ASSERT(c_p->flags & F_OFF_HEAP_MSGQ_CHNG);
-    ASSERT(erts_atomic32_read_nob(&c_p->state) & ERTS_PSFLG_OFF_HEAP_MSGQ);
-
-    /*
-     * This job was first initiated when the process changed to off heap
-     * message queue management. Since then ERTS_PSFLG_OFF_HEAP_MSGQ
-     * has been set. However, the management state might have been changed
-     * again (multiple times) since then. Check users last requested state
-     * (the flags F_OFF_HEAP_MSGQ, and F_ON_HEAP_MSGQ), and make the state
-     * consistent with that.
-     */
-
-    if (!(c_p->flags & F_OFF_HEAP_MSGQ))
-	erts_atomic32_read_band_nob(&c_p->state,
-					~ERTS_PSFLG_OFF_HEAP_MSGQ);
-    else {
-	reds += 2;
-	erts_proc_lock(c_p, ERTS_PROC_LOCK_MSGQ);
-        erts_proc_sig_fetch(c_p);
-	erts_proc_unlock(c_p, ERTS_PROC_LOCK_MSGQ);
-	reds += erts_move_messages_off_heap(c_p);
-    }
-    c_p->flags &= ~F_OFF_HEAP_MSGQ_CHNG;
-    return reds;
-}
-
 typedef struct {
     Eterm pid;
     ErtsThrPrgrLaterOp lop;
@@ -1020,15 +949,11 @@ change_off_heap_msgq(void *vcohmq)
     ErtsChangeOffHeapMessageQueue *cohmq;
     /*
      * Now we've waited thread progress which ensures that all
-     * messages to the process are enqueued off heap. Schedule
-     * completion of this change as a system task on the process
-     * itself. This in order to avoid lock contention on its
-     * main lock. We will be called in
-     * erts_complete_off_heap_message_queue_change() (above) when
-     * the system task has been selected for execution.
+     * messages to the process are enqueued off heap. Signal the
+     * process to complete this change itself.
      */
     cohmq = (ErtsChangeOffHeapMessageQueue *) vcohmq;
-    erts_schedule_complete_off_heap_message_queue_change(cohmq->pid);
+    erts_proc_sig_send_move_msgq_off_heap(NULL, cohmq->pid);
     erts_free(ERTS_ALC_T_MSGQ_CHNG, vcohmq);
 }
 
@@ -1038,12 +963,12 @@ erts_change_message_queue_management(Process *c_p, Eterm new_state)
     Eterm res;
 
 #ifdef DEBUG
-    if (c_p->flags & F_OFF_HEAP_MSGQ) {
+    if (c_p->sig_qs.flags & FS_OFF_HEAP_MSGQ) {
 	ASSERT(erts_atomic32_read_nob(&c_p->state)
 	       & ERTS_PSFLG_OFF_HEAP_MSGQ);
     }
     else {
-	if (c_p->flags & F_OFF_HEAP_MSGQ_CHNG) {
+	if (c_p->sig_qs.flags & FS_OFF_HEAP_MSGQ_CHNG) {
 	    ASSERT(erts_atomic32_read_nob(&c_p->state)
 		   & ERTS_PSFLG_OFF_HEAP_MSGQ);
 	}
@@ -1054,26 +979,26 @@ erts_change_message_queue_management(Process *c_p, Eterm new_state)
     }
 #endif
 
-    switch (c_p->flags & (F_OFF_HEAP_MSGQ|F_ON_HEAP_MSGQ)) {
+    switch (c_p->sig_qs.flags & (FS_OFF_HEAP_MSGQ|FS_ON_HEAP_MSGQ)) {
 
-    case F_OFF_HEAP_MSGQ:
+    case FS_OFF_HEAP_MSGQ:
 	res = am_off_heap;
 
 	switch (new_state) {
 	case am_off_heap:
 	    break;
 	case am_on_heap:
-	    c_p->flags |= F_ON_HEAP_MSGQ;
-	    c_p->flags &= ~F_OFF_HEAP_MSGQ;
+	    c_p->sig_qs.flags |= FS_ON_HEAP_MSGQ;
+	    c_p->sig_qs.flags &= ~FS_OFF_HEAP_MSGQ;
 	    /*
 	     * We are not allowed to clear ERTS_PSFLG_OFF_HEAP_MSGQ
 	     * if a off heap change is ongoing. It will be adjusted
 	     * when the change completes...
 	     */
-	    if (!(c_p->flags & F_OFF_HEAP_MSGQ_CHNG)) {
+	    if (!(c_p->sig_qs.flags & FS_OFF_HEAP_MSGQ_CHNG)) {
 		/* Safe to clear ERTS_PSFLG_OFF_HEAP_MSGQ... */
 		erts_atomic32_read_band_nob(&c_p->state,
-						~ERTS_PSFLG_OFF_HEAP_MSGQ);
+					    ~ERTS_PSFLG_OFF_HEAP_MSGQ);
 	    }
 	    break;
 	default:
@@ -1082,14 +1007,14 @@ erts_change_message_queue_management(Process *c_p, Eterm new_state)
 	}
 	break;
 
-    case F_ON_HEAP_MSGQ:
+    case FS_ON_HEAP_MSGQ:
 	res = am_on_heap;
 
 	switch (new_state) {
 	case am_on_heap:
 	    break;
 	case am_off_heap:
-	    c_p->flags &= ~F_ON_HEAP_MSGQ;
+	    c_p->sig_qs.flags &= ~FS_ON_HEAP_MSGQ;
 	    goto change_to_off_heap;
 	default:
 	    res = THE_NON_VALUE; /* badarg */
@@ -1107,13 +1032,13 @@ erts_change_message_queue_management(Process *c_p, Eterm new_state)
 
 change_to_off_heap:
 
-    c_p->flags |= F_OFF_HEAP_MSGQ;
+    c_p->sig_qs.flags |= FS_OFF_HEAP_MSGQ;
 
     /*
      * We do not have to schedule a change if
      * we have an ongoing off heap change...
      */
-    if (!(c_p->flags & F_OFF_HEAP_MSGQ_CHNG)) {
+    if (!(c_p->sig_qs.flags & FS_OFF_HEAP_MSGQ_CHNG)) {
 	ErtsChangeOffHeapMessageQueue *cohmq;
 	/*
 	 * Need to set ERTS_PSFLG_OFF_HEAP_MSGQ and wait
@@ -1124,8 +1049,8 @@ change_to_off_heap:
 	 * the message queue at all.
 	 */
 	erts_atomic32_read_bor_nob(&c_p->state,
-				       ERTS_PSFLG_OFF_HEAP_MSGQ);
-	c_p->flags |= F_OFF_HEAP_MSGQ_CHNG;
+				   ERTS_PSFLG_OFF_HEAP_MSGQ);
+	c_p->sig_qs.flags |= FS_OFF_HEAP_MSGQ_CHNG;
 	cohmq = erts_alloc(ERTS_ALC_T_MSGQ_CHNG,
 			   sizeof(ErtsChangeOffHeapMessageQueue));
 	cohmq->pid = c_p->common.id;
@@ -1582,6 +1507,15 @@ void erts_factory_undo(ErtsHeapFactory* factory)
                                    ERTS_HEAP_FRAG_SIZE(factory->heap_frags_saved->alloc_size));
                 }
             }
+            if (factory->message) {
+                ASSERT(factory->message->data.attached != ERTS_MSG_COMBINED_HFRAG);
+                ASSERT(!factory->message->data.heap_frag);
+
+                /* Set the message to NIL in order for it not to be treated as
+                   a distributed message by erts_cleanup_messages */
+                factory->message->m[0] = NIL;
+                erts_cleanup_messages(factory->message);
+            }
         }
         break;
 
