@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1999-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1999-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -684,7 +684,7 @@ trace_sched_aux(Process *p, ErtsProcLocks locks, Eterm what)
 	curr_func = 0;
     else {
 	if (!p->current)
-	    p->current = find_function_from_pc(p->i);
+	    p->current = erts_find_function_from_pc(p->i);
 	curr_func = p->current != NULL;
     }
 
@@ -830,7 +830,7 @@ trace_receive(Process* receiver,
 }
 
 int
-seq_trace_update_send(Process *p)
+seq_trace_update_serial(Process *p)
 {
     ErtsTracer seq_tracer = erts_get_system_seq_tracer();
     ASSERT((is_tuple(SEQ_TRACE_TOKEN(p)) || is_nil(SEQ_TRACE_TOKEN(p))));
@@ -853,6 +853,18 @@ seq_trace_update_send(Process *p)
     return 1;
 }
 
+void
+erts_seq_trace_update_node_token(Eterm token)
+{
+    Eterm serial;
+    Uint serial_num;
+    SEQ_TRACE_T_SENDER(token) = erts_this_dist_entry->sysname;
+    serial = SEQ_TRACE_T_SERIAL(token);
+    serial_num = unsigned_val(serial);
+    serial_num++;
+    SEQ_TRACE_T_SERIAL(token) = make_small(serial_num);
+}
+
 
 /* Send a sequential trace message to the sequential tracer.
  * p is the caller (which contains the trace token), 
@@ -929,16 +941,18 @@ seq_trace_output_generic(Eterm token, Eterm msg, Uint type,
 #undef LOCAL_HEAP_SIZE
 }
 
+
+
+
 /* Send {trace_ts, Pid, return_to, {Mod, Func, Arity}, Timestamp}
  * or   {trace, Pid, return_to, {Mod, Func, Arity}}
  */
 void 
-erts_trace_return_to(Process *p, BeamInstr *pc)
+erts_trace_return_to(Process *p, ErtsCodePtr pc)
 {
+    const ErtsCodeMFA *cmfa = erts_find_function_from_pc(pc);
     Eterm mfa;
 
-    ErtsCodeMFA *cmfa = find_function_from_pc(pc);
-
     if (!cmfa) {
 	mfa = am_undefined;
     } else {
@@ -1370,8 +1384,8 @@ trace_gc(Process *p, Eterm what, Uint size, Eterm msg)
 }
 
 void 
-monitor_long_schedule_proc(Process *p, ErtsCodeMFA *in_fp,
-                           ErtsCodeMFA *out_fp, Uint time)
+monitor_long_schedule_proc(Process *p, const ErtsCodeMFA *in_fp,
+                           const ErtsCodeMFA *out_fp, Uint time)
 {
     ErlHeapFragment *bp;
     ErlOffHeap *off_heap;
@@ -1924,7 +1938,7 @@ profile_runnable_proc(Process *p, Eterm status){
     Eterm *hp, msg;
     Eterm where = am_undefined;
     ErlHeapFragment *bp = NULL;
-    ErtsCodeMFA *cmfa = NULL;
+    const ErtsCodeMFA *cmfa = NULL;
 
     ErtsThrPrgrDelayHandle dhndl;
     Uint hsz = 4 + 6 + patch_ts_size(erts_system_profile_ts_type)-1;
@@ -1938,7 +1952,7 @@ profile_runnable_proc(Process *p, Eterm status){
         if (p->current) {
             cmfa = p->current;
         } else {
-            cmfa = find_function_from_pc(p->i);
+            cmfa = erts_find_function_from_pc(p->i);
         }
     }
 
@@ -2118,36 +2132,49 @@ sys_msg_disp_failure(ErtsSysMsgQ *smqp, Eterm receiver)
 	erts_thr_progress_unblock();
 	break;
     case SYS_MSG_TYPE_ERRLGR: {
-	char *no_elgger = "(no logger present)";
 	Eterm *tp;
 	Eterm tag;
+
 	if (is_not_tuple(smqp->msg)) {
-	unexpected_elmsg:
-	    erts_fprintf(stderr,
-			 "%s unexpected logger message: %T\n",
-			 no_elgger,
-			 smqp->msg);
+	    goto unexpected_error_msg;
+	}
+	tp = tuple_val(smqp->msg);
+	if (arityval(tp[0]) != 2) {
+	    goto unexpected_error_msg;
+	}
+	if (is_not_tuple(tp[2])) {
+	    goto unexpected_error_msg;
+	}
+	tp = tuple_val(tp[2]);
+	if (arityval(tp[0]) != 3) {
+	    goto unexpected_error_msg;
+	}
+	tag = tp[1];
+	if (is_not_tuple(tp[3])) {
+	    goto unexpected_error_msg;
+	}
+	tp = tuple_val(tp[3]);
+	if (arityval(tp[0]) != 3) {
+	    goto unexpected_error_msg;
+	}
+	if (is_not_list(tp[3])) {
+	    goto unexpected_error_msg;
 	}
 
-	tp = tuple_val(smqp->msg);
-	if (arityval(tp[0]) != 2)
-	    goto unexpected_elmsg;
-	if (is_not_tuple(tp[2]))
-	    goto unexpected_elmsg;
-	tp = tuple_val(tp[2]);
-	if (arityval(tp[0]) != 3)
-	    goto unexpected_elmsg;
-	tag = tp[1];
-	if (is_not_tuple(tp[3]))
-	    goto unexpected_elmsg;
-	tp = tuple_val(tp[3]);
-	if (arityval(tp[0]) != 3)
-	    goto unexpected_elmsg;
-	if (is_not_list(tp[3]))
-	    goto unexpected_elmsg;
-	erts_fprintf(stderr, "%s %T: %T\n",
-		     no_elgger, tag, CAR(list_val(tp[3])));
-	break;
+        {
+            static const char *no_logger = "(no logger present)";
+        /* no_error_logger: */
+            erts_fprintf(stderr, "%s %T: %T\n",
+                         no_logger, tag, CAR(list_val(tp[3])));
+            break;
+        unexpected_error_msg:
+            erts_fprintf(stderr,
+                         "%s unexpected logger message: %T\n",
+                         no_logger,
+                         smqp->msg);
+            break;
+        }
+        ASSERT(0);
     }
     case SYS_MSG_TYPE_PROC_MSG:
         break;
@@ -2215,7 +2242,7 @@ sys_msg_dispatcher_func(void *unused)
     callbacks.wait = sys_msg_dispatcher_wait;
     callbacks.finalize_wait = sys_msg_dispatcher_fin_wait;
 
-    tpd = erts_thr_progress_register_managed_thread(NULL, &callbacks, 0);
+    tpd = erts_thr_progress_register_managed_thread(NULL, &callbacks, 0, 0);
 
     while (1) {
 	int end_wait = 0;
@@ -2234,6 +2261,7 @@ sys_msg_dispatcher_func(void *unused)
 
 	/* Fetch current trace message queue ... */
 	if (!sys_message_queue) {
+            wait = 1;
 	    erts_mtx_unlock(&smq_mtx);
 	    end_wait = 1;
 	    erts_thr_progress_active(tpd, 0);
@@ -2241,8 +2269,23 @@ sys_msg_dispatcher_func(void *unused)
 	    erts_mtx_lock(&smq_mtx);
 	}
 
-	while (!sys_message_queue)
-	    erts_cnd_wait(&smq_cnd, &smq_mtx);
+	while (!sys_message_queue) {
+            if (wait)
+                erts_cnd_wait(&smq_cnd, &smq_mtx);
+            if (sys_message_queue)
+                break;
+            wait = 1;
+	    erts_mtx_unlock(&smq_mtx);
+            /*
+             * Ensure thread progress continue. We might have
+             * been the last thread to go to sleep. In that case
+             * erts_thr_progress_finalize_wait() will take care
+             * of it...
+             */
+	    erts_thr_progress_finalize_wait(tpd);
+	    erts_thr_progress_prepare_wait(tpd);
+	    erts_mtx_lock(&smq_mtx);
+        }
 
 	local_sys_message_queue = sys_message_queue;
 	sys_message_queue = NULL;
@@ -2818,6 +2861,8 @@ is_tracer_enabled(Process* c_p, ErtsProcLocks c_p_locks,
                   enum ErtsTracerOpt topt, Eterm tag) {
     Eterm nif_result;
 
+    ASSERT(t_p);
+
 #if defined(ERTS_ENABLE_LOCK_CHECK)
     if (c_p)
         ERTS_LC_ASSERT(erts_proc_lc_my_proc_locks(c_p) == c_p_locks
