@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -23,13 +23,29 @@
 -module(ssh_compat_SUITE).
 
 -include_lib("common_test/include/ct.hrl").
--include_lib("ssh/src/ssh_transport.hrl"). % #ssh_msg_kexinit{}
+-include("ssh_transport.hrl"). % #ssh_msg_kexinit{}
 -include_lib("kernel/include/inet.hrl"). % #hostent{}
 -include_lib("kernel/include/file.hrl"). % #file_info{}
 -include("ssh_test_lib.hrl").
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
+-export([
+         suite/0,
+         all/0,
+         groups/0,
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_group/2,
+         end_per_group/2
+        ]).
+
+-export([
+         all_algorithms_sftp_exec_reneg_otp_is_client/1,
+         check_docker_present/1,
+         login_otp_is_client/1,
+         login_otp_is_server/1,
+         renegotiation_otp_is_server/1,
+         send_recv_big_with_renegotiate_otp_is_client/1
+        ]).
 
 -define(USER,"sshtester").
 -define(PASSWD, "foobar").
@@ -41,7 +57,7 @@
 %%--------------------------------------------------------------------
 
 suite() ->
-    [{timetrap,{seconds,60}}].
+    [{timetrap,{seconds,90}}].
 
 all() ->
 %%    [check_docker_present] ++
@@ -53,7 +69,7 @@ groups() ->
                        send_recv_big_with_renegotiate_otp_is_client
                       ]},
      {otp_server, [], [login_otp_is_server,
-                       all_algorithms_sftp_exec_reneg_otp_is_server
+                       renegotiation_otp_is_server
                       ]} |
      [{G, [], [{group,otp_client}, {group,otp_server}]} || G <- ssh_image_versions()]
     ].
@@ -86,13 +102,13 @@ init_per_suite(Config) ->
                Config
        end).
 
-end_per_suite(Config) ->
+end_per_suite(_Config) ->
     %% Remove all containers that are not running:
 %%%    os:cmd("docker rm $(docker ps -aq -f status=exited)"),
     %% Remove dangling images:
 %%%    os:cmd("docker rmi $(docker images -f dangling=true -q)"),
     catch ssh:stop(),
-    Config.
+    ok.
 
 
 init_per_group(otp_server, Config) ->
@@ -150,8 +166,7 @@ init_per_group(G, Config0) ->
                             stop_docker(ID),
                             {fail, "Can't contact docker sshd"}
                     catch
-                        Class:Exc ->
-                            ST = erlang:get_stacktrace(),
+                        Class:Exc:ST ->
                             ct:log("common_algs: ~p:~p~n~p",[Class,Exc,ST]),
                             stop_docker(ID),
                             {fail, "Failed during setup"}
@@ -160,8 +175,7 @@ init_per_group(G, Config0) ->
                 cant_start_docker ->
                     {skip, "Can't start docker"};
 
-                C:E ->
-                    ST = erlang:get_stacktrace(),
+                C:E:ST ->
                     ct:log("No ~p~n~p:~p~n~p",[G,C,E,ST]),
                     {skip, "Can't start docker"}
             end;
@@ -198,17 +212,19 @@ login_otp_is_client(Config) ->
                                             [' ']
                                     end
         ],
-                                        
+
     chk_all_algos(?FUNCTION_NAME, CommonAuths, Config,
                   fun(AuthMethod,Alg) ->
                           {Opts,Dir} =
                               case AuthMethod of
                                   publickey ->
-                                      {[], setup_remote_auth_keys_and_local_priv(Alg, Config)};
+                                      {[{pref_public_key_algs, [Alg]}],
+                                       setup_remote_auth_keys_and_local_priv(Alg, Config)};
                                   _ ->
                                       {[{password,?PASSWD}], new_dir(Config)}
                               end,
                           ssh:connect(IP, Port, [{auth_methods, atom_to_list(AuthMethod)},
+                                                 {preferred_algorithms, ssh_transport:supported_algorithms()},
                                                  {user,?USER},
                                                  {user_dir, Dir},
                                                  {silently_accept_hosts,true},
@@ -236,7 +252,9 @@ login_otp_is_server(Config) ->
                           {Opts,UsrDir} =
                               case AuthMethod of
                                   publickey ->
-                                      {[{user_passwords, [{?USER,?BAD_PASSWD}]}],
+                                      {[{user_passwords, [{?USER,?BAD_PASSWD}]},
+                                        {pref_public_key_algs, [Alg]}
+                                       ],
                                        setup_remote_priv_and_local_auth_keys(Alg, Config)
                                       };
                                   _ ->
@@ -247,6 +265,7 @@ login_otp_is_server(Config) ->
                           {Server, Host, HostPort} =
                               ssh_test_lib:daemon(0,
                                                   [{auth_methods, atom_to_list(AuthMethod)},
+                                                   {preferred_algorithms, ssh_transport:supported_algorithms()},
                                                    {system_dir, SysDir},
                                                    {user_dir, UsrDir},
                                                    {failfun, fun ssh_test_lib:failfun/2}
@@ -266,13 +285,16 @@ all_algorithms_sftp_exec_reneg_otp_is_client(Config) ->
     {IP,Port} = ip_port(Config),
     chk_all_algos(?FUNCTION_NAME, CommonAlgs, Config,
                   fun(Tag, Alg) ->
+                          PrefAlgs =
+                              [{T,L} || {T,L} <- ssh_transport:supported_algorithms(),
+                                        T =/= Tag],
                           ConnRes =
                               ssh:connect(IP, Port, 
                                           [{user,?USER},
                                            {password,?PASSWD},
                                            {auth_methods, "password"},
                                            {user_dir, new_dir(Config)},
-                                           {preferred_algorithms, [{Tag,[Alg]}]},
+                                           {preferred_algorithms, [{Tag,[Alg]} | PrefAlgs]},
                                            {silently_accept_hosts,true},
                                            {user_interaction,false}
                                           ])  ,
@@ -286,43 +308,59 @@ all_algorithms_sftp_exec_reneg_otp_is_client(Config) ->
                   end).
 
 %%--------------------------------------------------------------------
-all_algorithms_sftp_exec_reneg_otp_is_server(Config) ->
-    CommonAlgs = proplists:get_value(common_remote_client_algs, Config),
-    UserDir = setup_remote_priv_and_local_auth_keys('ssh-rsa', Config),
-    chk_all_algos(?FUNCTION_NAME, CommonAlgs, Config,
-                  fun(Tag,Alg) ->
-                          HostKeyAlg = case Tag of
-                                           public_key -> Alg;
-                                           _ -> 'ssh-rsa'
-                                       end,
-                          SftpRootDir = new_dir(Config),
-                          %% ct:log("Rootdir = ~p",[SftpRootDir]),
-                          {Server, Host, HostPort} =
-                              ssh_test_lib:daemon(0,
-                                                  [{preferred_algorithms, [{Tag,[Alg]}]},
-                                                   {system_dir, setup_local_hostdir(HostKeyAlg, Config)},
-                                                   {user_dir, UserDir},
-                                                   {user_passwords, [{?USER,?PASSWD}]},
-                                                   {failfun, fun ssh_test_lib:failfun/2},
-                                                   {subsystems,
-                                                    [ssh_sftpd:subsystem_spec([{cwd,SftpRootDir},
-                                                                               {root,SftpRootDir}]),
-                                                     {"echo_10",{ssh_echo_server,[10,[{dbg,true}]]}}
-                                                    ]}
-                                                  ]),
-                          R = do([fun() ->
-                                          exec_from_docker(Config, Host, HostPort,
-                                                           "hi_there.\r\n",
-                                                           [<<"hi_there">>],
-                                                           "")
-                                  end,
-                                  fun() ->
-                                          sftp_tests_erl_server(Config, Host, HostPort, SftpRootDir, UserDir)
-                                  end
-                                 ]),
-                          ssh:stop_daemon(Server),
-                          R
-                  end).
+renegotiation_otp_is_server(Config) ->
+    PublicKeyAlgs = [A || {public_key,A} <- proplists:get_value(common_remote_client_algs, Config, [])],
+    ct:log("PublicKeyAlgs = ~p", [PublicKeyAlgs]),
+    UserDir = setup_remote_priv_and_local_auth_keys(hd(PublicKeyAlgs), Config),
+    SftpRootDir = new_dir(Config),
+    ct:log("Rootdir = ~p",[SftpRootDir]),
+    Parent = self(),
+    Ref = make_ref(),
+    {Server, Host, Port} =
+        ssh_test_lib:daemon(0,
+                            [{system_dir, setup_local_hostdir(Config)},
+                             {user_dir, UserDir},
+                             {user_passwords, [{?USER,?PASSWD}]},
+                             {failfun, fun ssh_test_lib:failfun/2},
+                             {modify_algorithms, [{append, [{public_key,PublicKeyAlgs}]}]},
+                             {connectfun,
+                              fun(_,_,_) ->
+                                      HostConnRef = self(),
+                                      reneg_tester(Parent, Ref, HostConnRef),
+                                      ct:log("Connected ~p",[HostConnRef]),
+                                      timer:sleep(1100), % Just a bit more than  1 s
+                                      ok
+                              end},
+                             {subsystems,
+                              [ssh_sftpd:subsystem_spec([{cwd,SftpRootDir},
+                                                         {root,SftpRootDir}])
+                              ]}
+                            ]),
+    case sftp_tests_erl_server(Config, Host, Port, SftpRootDir, UserDir, Ref) of
+        ok ->
+            ssh:stop_daemon(Server);
+        {error,Error} ->
+            ssh:stop_daemon(Server),
+            ct:log("Error: ~p", [Error]),
+            {fail, Error}
+    end.
+
+
+reneg_tester(Parent, Ref, HostConnRef) ->
+    spawn(fun() ->
+                  reneg_tester_loop(Parent, Ref, HostConnRef, renegotiate_test(init,HostConnRef))
+          end).
+                  
+reneg_tester_loop(Parent, Ref, HostConnRef, Kex1) ->  
+    case ssh_test_lib:get_kex_init(HostConnRef) of
+        Kex1 ->
+            timer:sleep(500),
+            reneg_tester_loop(Parent, Ref, HostConnRef, Kex1);
+        _OtherKex ->
+            ct:log("Kex is changed.", []),
+            Parent ! {kex_changed, Ref}
+    end.
+
 
 %%--------------------------------------------------------------------
 send_recv_big_with_renegotiate_otp_is_client(Config) ->
@@ -332,6 +370,7 @@ send_recv_big_with_renegotiate_otp_is_client(Config) ->
                                     {password,?PASSWD},
                                     {user_dir, setup_remote_auth_keys_and_local_priv('ssh-rsa', Config)},
                                     {silently_accept_hosts,true},
+                                    {preferred_algorithms, ssh_transport:supported_algorithms()},
                                     {user_interaction,false}
                                    ]),
 
@@ -440,6 +479,7 @@ exec_from_docker(Config, HostIP, HostPort, Command, Expects, ExtraSshArg) when i
                          [{user,?USER},
                           {password,?PASSWD},
                           {user_dir, new_dir(Config)},
+                          {preferred_algorithms, ssh_transport:supported_algorithms()},
                           {silently_accept_hosts,true},
                           {user_interaction,false}
                          ]),
@@ -523,21 +563,22 @@ result_of_exec(C, Ch, ExitStatus, Acc) ->
 chk_all_algos(FunctionName, CommonAlgs, Config, DoTestFun) when is_function(DoTestFun,2) ->
     ct:comment("~p algorithms",[length(CommonAlgs)]),
     %% Check each algorithm
-    Failed =
+    Nmax = length(CommonAlgs),
+    {_N,Failed} =
         lists:foldl(
-          fun({Tag,Alg}, FailedAlgos) ->
-                  %% ct:log("Try ~p",[Alg]),
+          fun({Tag,Alg}, {N,FailedAlgos}) ->
+                  ct:log("Try ~p  ~p/~p",[{Tag,Alg},N,Nmax]),
                   case DoTestFun(Tag,Alg) of
                       {ok,C} ->
                           ssh:close(C),
-                          FailedAlgos;
+                          {N+1,FailedAlgos};
                       ok ->
-                          FailedAlgos;
+                          {N+1,FailedAlgos};
                       Other ->
                           ct:log("FAILED! ~p ~p: ~p",[Tag,Alg,Other]),
-                          [{Alg,Other}|FailedAlgos]
+                          {N+1, [{Alg,Other}|FailedAlgos]}
                   end
-          end, [], CommonAlgs),
+          end, {1,[]}, CommonAlgs),
     ct:pal("~s", [format_result_table_use_all_algos(FunctionName, Config, CommonAlgs, Failed)]),
     case Failed of
         [] ->
@@ -548,39 +589,27 @@ chk_all_algos(FunctionName, CommonAlgs, Config, DoTestFun) when is_function(DoTe
 
 
 
-%%%----------------------------------------------------------------
-%%%
-%%% Call all Funs as Fun() which returns 'ok', {ok,C} or Other.
-%%% do/1 returns 'ok' or the first encountered value that is not
-%%% successful.
-%%%
-
-do(Funs) ->
-    do(Funs, 1).
-
-do([Fun|Funs], N) ->
-    case Fun() of
-        ok ->
-            %% ct:log("Fun ~p ok",[N]),
-            do(Funs, N-1);
-        {ok,C} ->
-            %% ct:log("Fun ~p {ok,C}",[N]),
-            ssh:close(C),
-            do(Funs, N-1);
-        Other ->
-            ct:log("Fun ~p FAILED:~n~p",[N, Other]),
-            Other
-    end;
-
-do([], _) ->
-    %% ct:log("All Funs ok",[]),
-    ok.
-
 %%--------------------------------------------------------------------
 %%
 %% Functions to set up local and remote host's and user's keys and directories
 %% 
 
+setup_local_hostdir(Config) ->
+    KeyAlgs = lists:usort(
+                [A || From <- [common_remote_client_algs,
+                               common_remote_server_algs],
+                      {public_key,A} <- proplists:get_value(From, Config, [])]),
+    setup_local_hostdirs(KeyAlgs, new_dir(Config), Config).
+
+
+setup_local_hostdirs(KeyAlgs, HostDir, Config) ->
+    lists:foreach(
+      fun(KeyAlg) ->
+              setup_local_hostdir(KeyAlg, HostDir, Config)
+      end, KeyAlgs),
+    HostDir.
+
+
 setup_local_hostdir(KeyAlg, Config) ->
     setup_local_hostdir(KeyAlg, new_dir(Config), Config).
 setup_local_hostdir(KeyAlg, HostDir, Config) ->
@@ -596,13 +625,6 @@ setup_remote_auth_keys_and_local_priv(KeyAlg, Config) ->
     {IP,Port} = ip_port(Config),
     setup_remote_auth_keys_and_local_priv(KeyAlg, IP, Port, new_dir(Config), Config).
 
-setup_remote_auth_keys_and_local_priv(KeyAlg, UserDir, Config) ->
-    {IP,Port} = ip_port(Config),
-    setup_remote_auth_keys_and_local_priv(KeyAlg, IP, Port, UserDir, Config).
-
-setup_remote_auth_keys_and_local_priv(KeyAlg, IP, Port, Config) ->
-    setup_remote_auth_keys_and_local_priv(KeyAlg, IP, Port, new_dir(Config), Config).
-
 setup_remote_auth_keys_and_local_priv(KeyAlg, IP, Port, UserDir, Config) ->
     {ok, {Priv,Publ}} = user_priv_pub_keys(Config, KeyAlg),
     %% Local private and public keys
@@ -614,6 +636,7 @@ setup_remote_auth_keys_and_local_priv(KeyAlg, IP, Port, UserDir, Config) ->
                                                    {password, ?PASSWD   },
                                                    {auth_methods, "password"},
                                                    {silently_accept_hosts,true},
+                                                   {preferred_algorithms, ssh_transport:supported_algorithms()},
                                                    {user_interaction,false}
                                                   ]),
     _ = ssh_sftp:make_dir(Ch, ".ssh"),
@@ -629,13 +652,6 @@ setup_remote_priv_and_local_auth_keys(KeyAlg, Config) ->
     {IP,Port} = ip_port(Config),
     setup_remote_priv_and_local_auth_keys(KeyAlg, IP, Port, new_dir(Config), Config).
 
-setup_remote_priv_and_local_auth_keys(KeyAlg, UserDir, Config) ->
-    {IP,Port} = ip_port(Config),
-    setup_remote_priv_and_local_auth_keys(KeyAlg, IP, Port, UserDir, Config).
-
-setup_remote_priv_and_local_auth_keys(KeyAlg, IP, Port, Config) ->
-    setup_remote_priv_and_local_auth_keys(KeyAlg, IP, Port, new_dir(Config), Config).
-
 setup_remote_priv_and_local_auth_keys(KeyAlg, IP, Port, UserDir, Config) ->
     {ok, {Priv,Publ}} = user_priv_pub_keys(Config, KeyAlg),
     %% Local auth_methods with public key
@@ -645,9 +661,11 @@ setup_remote_priv_and_local_auth_keys(KeyAlg, IP, Port, UserDir, Config) ->
     {ok,Ch,Cc} = ssh_sftp:start_channel(IP, Port, [{user,     ?USER  },
                                                    {password, ?PASSWD   },
                                                    {auth_methods, "password"},
+                                                   {preferred_algorithms, ssh_transport:supported_algorithms()},
                                                    {silently_accept_hosts,true},
                                                    {user_interaction,false}
                                                   ]),
+    rm_id_in_remote_dir(Ch, ".ssh"),
     _ = ssh_sftp:make_dir(Ch, ".ssh"),
     DstFile = filename:join(".ssh", dst_filename(user,KeyAlg)),
     ok = ssh_sftp:write_file(Ch, DstFile, Priv),
@@ -658,6 +676,18 @@ setup_remote_priv_and_local_auth_keys(KeyAlg, IP, Port, UserDir, Config) ->
     ok = ssh:close(Cc),
     UserDir.
 
+rm_id_in_remote_dir(Ch, Dir) ->
+    case ssh_sftp:list_dir(Ch, Dir) of
+        {error,_Error} ->
+            ok;
+        {ok,FileNames} ->
+            lists:foreach(fun("id_"++_ = F) ->
+                                  ok = ssh_sftp:delete(Ch, filename:join(Dir,F));
+                             (_) ->
+                                  leave
+                          end, FileNames)
+    end.
+
 user_priv_pub_keys(Config, KeyAlg) -> priv_pub_keys("users_keys", user, Config, KeyAlg).
 host_priv_pub_keys(Config, KeyAlg) -> priv_pub_keys("host_keys",  host, Config, KeyAlg).
 
@@ -673,6 +703,8 @@ src_filename(user, 'ssh-rsa'            ) -> "id_rsa";
 src_filename(user, 'rsa-sha2-256'       ) -> "id_rsa";
 src_filename(user, 'rsa-sha2-512'       ) -> "id_rsa";
 src_filename(user, 'ssh-dss'            ) -> "id_dsa";
+src_filename(user, 'ssh-ed25519'        ) -> "id_ed25519";
+src_filename(user, 'ssh-ed448'          ) -> "id_ed448";
 src_filename(user, 'ecdsa-sha2-nistp256') -> "id_ecdsa256";
 src_filename(user, 'ecdsa-sha2-nistp384') -> "id_ecdsa384";
 src_filename(user, 'ecdsa-sha2-nistp521') -> "id_ecdsa521";
@@ -680,6 +712,8 @@ src_filename(host, 'ssh-rsa'            ) -> "ssh_host_rsa_key";
 src_filename(host, 'rsa-sha2-256'       ) -> "ssh_host_rsa_key";
 src_filename(host, 'rsa-sha2-512'       ) -> "ssh_host_rsa_key";
 src_filename(host, 'ssh-dss'            ) -> "ssh_host_dsa_key";
+src_filename(host, 'ssh-ed25519'        ) -> "ssh_host_ed25519_key";
+src_filename(host, 'ssh-ed448'          ) -> "ssh_host_ed448_key";
 src_filename(host, 'ecdsa-sha2-nistp256') -> "ssh_host_ecdsa_key256";
 src_filename(host, 'ecdsa-sha2-nistp384') -> "ssh_host_ecdsa_key384";
 src_filename(host, 'ecdsa-sha2-nistp521') -> "ssh_host_ecdsa_key521".
@@ -688,6 +722,8 @@ dst_filename(user, 'ssh-rsa'            ) -> "id_rsa";
 dst_filename(user, 'rsa-sha2-256'       ) -> "id_rsa";
 dst_filename(user, 'rsa-sha2-512'       ) -> "id_rsa";
 dst_filename(user, 'ssh-dss'            ) -> "id_dsa";
+dst_filename(user, 'ssh-ed25519'        ) -> "id_ed25519";
+dst_filename(user, 'ssh-ed448'          ) -> "id_ed448";
 dst_filename(user, 'ecdsa-sha2-nistp256') -> "id_ecdsa";
 dst_filename(user, 'ecdsa-sha2-nistp384') -> "id_ecdsa";
 dst_filename(user, 'ecdsa-sha2-nistp521') -> "id_ecdsa";
@@ -695,6 +731,8 @@ dst_filename(host, 'ssh-rsa'            ) -> "ssh_host_rsa_key";
 dst_filename(host, 'rsa-sha2-256'       ) -> "ssh_host_rsa_key";
 dst_filename(host, 'rsa-sha2-512'       ) -> "ssh_host_rsa_key";
 dst_filename(host, 'ssh-dss'            ) -> "ssh_host_dsa_key";
+dst_filename(host, 'ssh-ed25519'        ) -> "ssh_host_ed25519_key";
+dst_filename(host, 'ssh-ed448'          ) -> "ssh_host_ed448_key";
 dst_filename(host, 'ecdsa-sha2-nistp256') -> "ssh_host_ecdsa_key";
 dst_filename(host, 'ecdsa-sha2-nistp384') -> "ssh_host_ecdsa_key";
 dst_filename(host, 'ecdsa-sha2-nistp521') -> "ssh_host_ecdsa_key".
@@ -770,11 +808,6 @@ ip_port(Config) ->
     {_Ver,{IP,Port},_} = proplists:get_value(id,Config),
     {IP,Port}.
 
-port_mapped_to(Id) ->
-    Cmnd = lists:concat(["docker ps --format \"{{.Ports}}\"  --filter id=",Id]),
-    [_, PortStr | _] = string:tokens(os:cmd(Cmnd), ":->/"),
-    list_to_integer(PortStr).
-
 ip(Id) ->
     Cmnd = lists:concat(["docker inspect --format='{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}}' ",
 			 Id]),
@@ -793,7 +826,7 @@ iptoa(IP) -> inet_parse:ntoa(IP).
 
 host_ip() ->
     {ok,Name} = inet:gethostname(),
-    {ok,#hostent{h_addr_list = [IP|_]}} = inet_res:gethostbyname(Name),
+    {ok,IP} = inet:ip(Name),
     IP.
 
 %%--------------------------------------------------------------------
@@ -814,29 +847,6 @@ new_dir(Config) ->
             new_dir(Config)
     end.
 
-clear_dir(Dir) ->
-    delete_all_contents(Dir),
-    {ok,[]} = file:list_dir(Dir),
-    Dir.
-
-delete_all_contents(Dir) ->
-    {ok,Fs} = file:list_dir(Dir),
-    lists:map(fun(F0) ->
-                      F = filename:join(Dir, F0),
-                      case filelib:is_file(F) of
-                          true ->
-                              file:delete(F);
-                          false ->
-                              case filelib:is_dir(F) of
-                                  true ->
-                                      delete_all_contents(F),
-                                      file:del_dir(F);
-                                  false ->
-                                      ct:log("Neither file nor dir: ~p",[F])
-                              end
-                      end
-              end, Fs).
-
 %%--------------------------------------------------------------------
 %%
 %% Find the intersection of algoritms for otp ssh and the docker ssh.
@@ -901,14 +911,25 @@ find_common_algs(Remote, Local) ->
 use_algorithms(RemoteHelloBin) ->
     MyAlgos = ssh:chk_algos_opts(
                 [{modify_algorithms,
-                  [{append,
-                    [{kex,['diffie-hellman-group1-sha1']}
-                    ]}
+                  [{append, alg_diff()}
                   ]}
                 ]),
     ssh_transport:adjust_algs_for_peer_version(binary_to_list(RemoteHelloBin)++"\r\n",
                                                MyAlgos).
 
+
+alg_diff() ->
+    alg_diff(ssh:default_algorithms(), ssh_transport:supported_algorithms()).
+
+alg_diff(L1, L2) when is_atom(hd(L1)) ; is_atom(hd(L2))  ->
+    (L2--L1)--['AEAD_AES_256_GCM','AEAD_AES_128_GCM'];
+alg_diff(L1, L2) ->
+    [{T, Diff} || {{T,EL1},{T,EL2}} <- lists:zip(L1,L2),
+                  Diff <- [alg_diff(EL1,EL2)],
+                  Diff =/= []
+    ].
+
+
 kexint_msg2default_algorithms(#ssh_msg_kexinit{kex_algorithms = Kex,
                                                server_host_key_algorithms = PubKey,
                                                encryption_algorithms_client_to_server = CipherC2S,
@@ -1005,8 +1026,7 @@ receive_hello(S) ->
         Result ->
             Result
     catch
-        Class:Error ->
-            ST = erlang:get_stacktrace(),
+        Class:Error:ST ->
             {error, {Class,Error,ST}}
     end.
         
@@ -1077,15 +1097,18 @@ receive_kexinit(S, Ack) ->
 %%% Test of sftp from the OpenSSH client side
 %%%
 
-sftp_tests_erl_server(Config, ServerIP, ServerPort, ServerRootDir, UserDir) ->
+sftp_tests_erl_server(Config, ServerIP, ServerPort, ServerRootDir, UserDir, Ref) ->
     try
         Cmnds = prepare_local_directory(ServerRootDir),
-        call_sftp_in_docker(Config, ServerIP, ServerPort, Cmnds, UserDir),
-        check_local_directory(ServerRootDir)
+        case call_sftp_in_docker(Config, ServerIP, ServerPort, Cmnds, UserDir, Ref) of
+            ok ->
+                check_local_directory(ServerRootDir);
+            {error,Error} ->
+                {error,Error}
+        end
     catch
-        Class:Error ->
-            ST = erlang:get_stacktrace(),
-            {error, {Class,Error,ST}}
+        Class:Excep:ST ->
+            {error, {Class,Excep,ST}}
     end.
 
 
@@ -1105,7 +1128,24 @@ prepare_local_directory(ServerRootDir) ->
      "chmod 222 unreadable_file",
      "exit"].
 
+
 check_local_directory(ServerRootDir) ->
+    TimesToTry = 3,  % sleep 0.5, 1, 2 and then 4 secs (7.5s in total)
+    check_local_directory(ServerRootDir, 500, TimesToTry-1).
+
+check_local_directory(ServerRootDir, SleepTime, N) ->
+    case do_check_local_directory(ServerRootDir) of
+        {error,_Error} when N>0 ->
+            %% Could be that the erlang side is faster and the docker's operations
+            %% are not yet finalized.
+            %% Sleep for a while and retry a few times:
+            timer:sleep(SleepTime),
+            check_local_directory(ServerRootDir, 2*SleepTime, N-1);
+        Other ->
+            Other
+    end.
+
+do_check_local_directory(ServerRootDir) ->
     case lists:sort(ok(file:list_dir(ServerRootDir)) -- [".",".."]) of
         ["ex_tst1","mydir","tst2"] ->
             {ok,Expect} = file:read_file(filename:join(ServerRootDir,"ex_tst1")),
@@ -1140,12 +1180,15 @@ check_local_directory(ServerRootDir) ->
             {error,{bad_dir_contents,"/"}}
     end.
 
-call_sftp_in_docker(Config, ServerIP, ServerPort, Cmnds, UserDir) ->
+
+call_sftp_in_docker(Config, ServerIP, ServerPort, Cmnds, UserDir, Ref) ->
     {DockerIP,DockerPort} = ip_port(Config),
+    ct:log("Going to connect ~p:~p", [DockerIP, DockerPort]),
     {ok,C} = ssh:connect(DockerIP, DockerPort,
                          [{user,?USER},
                           {password,?PASSWD},
                           {user_dir, UserDir},
+                          {preferred_algorithms, ssh_transport:supported_algorithms()},
                           {silently_accept_hosts,true},
                           {user_interaction,false}
                          ]),
@@ -1158,7 +1201,7 @@ call_sftp_in_docker(Config, ServerIP, ServerPort, Cmnds, UserDir) ->
     PostExpectCmnds= [],
     ExpectCmnds = 
         PreExpectCmnds ++
-        ["expect \"sftp>\" {send \""++Cmnd++"\n\"}\n" || Cmnd <- Cmnds] ++
+        ["expect \"sftp>\" {sleep 1; send \""++Cmnd++"\n\"}\n" || Cmnd <- Cmnds] ++
         PostExpectCmnds,
     
     %% Make an commands file in the docker
@@ -1168,26 +1211,48 @@ call_sftp_in_docker(Config, ServerIP, ServerPort, Cmnds, UserDir) ->
 
     %% Call expect in the docker
     {ok, Ch1} = ssh_connection:session_channel(C, infinity),
-    Kex1 = renegotiate_test(init, C),
+    ct:log("Going to execute 'expect commands' in docker", []),
     success = ssh_connection:exec(C, Ch1, "expect commands", infinity),
+    ct:log("'expect commands' in docker executed!", []),
+    case recv_log_msgs(C, Ch1) of
+        ok ->
+            receive
+                {kex_changed, Ref} ->
+                    %% success
+                    ct:log("Kex changed",[]),
+                    ssh:close(C),
+                    ok
+            after
+                30000 ->
+                    %% failure
+                    ct:log("Kex NOT changed",[]),
+                    ssh:close(C),
+                    {error,"Kex NOT changed"}
+            end;
 
-    renegotiate_test(Kex1, C),
-    recv_log_msgs(C, Ch1),
+        {error,Error} ->
+            ssh:close(C),
+            {error,Error}
+    end.
 
-    %% Done.
-    ssh:close(C).
 
 recv_log_msgs(C, Ch) ->
     receive
         {ssh_cm,C,{closed,Ch}} ->
-            %% ct:log("Channel closed ~p",[{closed,1}]),
+            ct:log("Channel closed ~p",[{closed,1}]),
             ok;
         {ssh_cm,C,{data,Ch,1,Msg}} ->
             ct:log("*** ERROR from docker:~n~s",[Msg]),
             recv_log_msgs(C, Ch);
         {ssh_cm,C,_Msg} ->
-            %% ct:log("Got ~p",[_Msg]),
+            ct:log("Got ~p",[_Msg]),
             recv_log_msgs(C, Ch)
+    after
+        30000 ->
+            %% failure
+            ct:log("Exec Channel ~p in ~p NOT closed properly", [Ch,C]),
+            ssh:close(C),
+            {error,"Exec channel NOT closed"}
     end.
 
 %%%----------------------------------------------------------------
@@ -1252,7 +1317,7 @@ one_test_erl_client(exec, Id, C) ->
         {eof,<<"Hi there\n">>} ->
             ok;
         Other ->
-            ct:pal("exec Got other ~p", [Other]),
+            ct:log("exec Got other ~p", [Other]),
             {error, {exec,Id,bad_msg,Other,undefined}}
     end;
 
@@ -1262,7 +1327,7 @@ one_test_erl_client(no_subsyst, Id, C) ->
         failure ->
             ok;
         Other ->
-            ct:pal("no_subsyst Got other ~p", [Other]),
+            ct:log("no_subsyst Got other ~p", [Other]),
             {error, {no_subsyst,Id,bad_ret,Other,undefined}}
     end;
 
@@ -1290,7 +1355,7 @@ one_test_erl_client(setenv, Id, C) ->
         {eof,Env} ->
             ok;
         Other ->
-            ct:pal("setenv Got other ~p", [Other]),
+            ct:log("setenv Got other ~p", [Other]),
             {error, {setenv,Id,bad_msg,Other,undefined}}
     end;
 
@@ -1308,8 +1373,7 @@ one_test_erl_client(SFTP, Id, C) when SFTP==sftp ; SFTP==sftp_async ->
         catch ssh_sftp:stop_channel(Ch),
         R
     catch
-        Class:Error ->
-            ST = erlang:get_stacktrace(),
+        Class:Error:ST ->
             {error, {SFTP,Id,Class,Error,ST}}
     end.
 
