@@ -50,6 +50,13 @@
 
 -define(WAIT_FOR_SHUTDOWN, 500).
 
+-define(SSHC_SUP(Pid), {sshc_sup, Pid, supervisor, [supervisor]}).
+-define(SSHD_SUP(Pid), {sshd_sup, Pid, supervisor, [supervisor]}).
+-define(SYSTEM_SUP(Pid,Address), {{ssh_system_sup, Address}, Pid, supervisor,[ssh_system_sup]}).
+-define(SUB_SYSTEM_SUP(Pid), {_,Pid, supervisor,[ssh_subsystem_sup]}).
+-define(ACCEPTOR_SUP(Pid,Address), {{ssh_acceptor_sup,Address},Pid,supervisor,[ssh_acceptor_sup]}).
+-define(ACCEPTOR_WORKER(Pid,Address), {{ssh_acceptor_sup,Address},Pid,worker,[ssh_acceptor]}).
+
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
 %%--------------------------------------------------------------------
@@ -110,11 +117,9 @@ end_per_testcase(_, _Config) ->
 default_tree(Config) when is_list(Config) ->
     TopSupChildren = supervisor:which_children(ssh_sup),
     2 = length(TopSupChildren),
-    {value, {sshc_sup, _, supervisor,[sshc_sup]}} =
-	lists:keysearch(sshc_sup, 1, TopSupChildren),
-    {value, {sshd_sup, _,supervisor,[sshd_sup]}} = 
-	lists:keysearch(sshd_sup, 1, TopSupChildren),
-    ?wait_match([{client_controller,_,worker,_}], supervisor:which_children(sshc_sup)),
+    {value, ?SSHC_SUP(_)} = lists:keysearch(sshc_sup, 1, TopSupChildren),
+    {value, ?SSHD_SUP(_)} = lists:keysearch(sshd_sup, 1, TopSupChildren),
+    ?wait_match([], supervisor:which_children(sshc_sup)),
     ?wait_match([], supervisor:which_children(sshd_sup)).
 
 %%-------------------------------------------------------------------------
@@ -122,15 +127,13 @@ sshc_subtree(Config) when is_list(Config) ->
     {_Pid, Host, Port} = proplists:get_value(server, Config),
     UserDir = proplists:get_value(userdir, Config),
 
-    ?wait_match([{client_controller,_,worker,_}], supervisor:which_children(sshc_sup)),
+    ?wait_match([], supervisor:which_children(sshc_sup)),
 
     {ok, Pid1} = ssh:connect(Host, Port, [{silently_accept_hosts, true},
 					  {user_interaction, false},
 					  {user, ?USER}, {password, ?PASSWD},{user_dir, UserDir}]),
 
-    ?wait_match([{{client,ssh_system_sup, LocalIP, LocalPort, ?DEFAULT_PROFILE},
-                  SysSup, supervisor,[ssh_system_sup]},
-                 {client_controller,_,worker,_}
+    ?wait_match([?SYSTEM_SUP(SysSup, #address{address=LocalIP, port=LocalPort, profile=?DEFAULT_PROFILE})
                 ],
 		supervisor:which_children(sshc_sup),
                 [SysSup, LocalIP, LocalPort]),
@@ -139,19 +142,17 @@ sshc_subtree(Config) when is_list(Config) ->
     {ok, Pid2} = ssh:connect(Host, Port, [{silently_accept_hosts, true},
 					  {user_interaction, false},
 					  {user, ?USER}, {password, ?PASSWD}, {user_dir, UserDir}]),
-    ?wait_match([{_, _,supervisor,[ssh_system_sup]},
-                 {_, _,supervisor,[ssh_system_sup]},
-                 {client_controller,_,worker,_}
+    ?wait_match([?SYSTEM_SUP(_,_),
+                 ?SYSTEM_SUP(_,_)
                 ],
 		supervisor:which_children(sshc_sup)),
 
     ssh:close(Pid1),
-    ?wait_match([{_, _,supervisor,[ssh_system_sup]},
-                 {client_controller,_,worker,_}
+    ?wait_match([?SYSTEM_SUP(_,_)
                 ],
 		supervisor:which_children(sshc_sup)),
     ssh:close(Pid2),
-    ?wait_match([{client_controller,_,worker,_}], supervisor:which_children(sshc_sup)).
+    ?wait_match([], supervisor:which_children(sshc_sup)).
 
 %%-------------------------------------------------------------------------
 sshd_subtree(Config) when is_list(Config) ->
@@ -162,9 +163,8 @@ sshd_subtree(Config) when is_list(Config) ->
                                                    [{?USER, ?PASSWD}]}]),
 
     ct:log("Expect HostIP=~p, Port=~p, Daemon=~p",[HostIP,Port,Daemon]),
-    ?wait_match([{{server,ssh_system_sup, ListenIP, Port, ?DEFAULT_PROFILE},
-		  Daemon, supervisor,
-		  [ssh_system_sup]}],
+    ?wait_match([?SYSTEM_SUP(Daemon, #address{address=ListenIP, port=Port, profile=?DEFAULT_PROFILE})
+                ],
 		supervisor:which_children(sshd_sup),
 		[ListenIP,Daemon]),
     true = ssh_test_lib:match_ip(HostIP, ListenIP),
@@ -184,9 +184,8 @@ sshd_subtree_profile(Config) when is_list(Config) ->
                                                    [{?USER, ?PASSWD}]},
                                                   {profile, Profile}]),
     ct:log("Expect HostIP=~p, Port=~p, Profile=~p, Daemon=~p",[HostIP,Port,Profile,Daemon]),
-    ?wait_match([{{server,ssh_system_sup, ListenIP,Port,Profile},
-		  Daemon, supervisor,
-		  [ssh_system_sup]}],
+    ?wait_match([?SYSTEM_SUP(Daemon, #address{address=ListenIP,port=Port,profile=Profile})
+                ],
 		supervisor:which_children(sshd_sup),
 		[ListenIP,Daemon]),
     true = ssh_test_lib:match_ip(HostIP, ListenIP),
@@ -301,20 +300,32 @@ shell_channel_tree(Config) ->
 						      {user_interaction, true},
 						      {user_dir, UserDir}]),
 
-    [ChannelSup|_] = Sups0 = chk_empty_con_daemon(Daemon),
+    [SubSysSup,_ChPid|_] = Sups0 = chk_empty_con_daemon(Daemon),
     
     {ok, ChannelId0} = ssh_connection:session_channel(ConnectionRef, infinity),
     ok = ssh_connection:shell(ConnectionRef,ChannelId0),
+    success = ssh_connection:ptty_alloc(ConnectionRef, ChannelId0, [{pty_opts,[{onlcr,1}]}]),
+
+    ?wait_match([{_,_, supervisor,[ssh_tcpip_forward_acceptor_sup]},
+                 {_,ChSup,supervisor,[ssh_channel_sup]},
+                 {connection,_,worker,[ssh_connection_handler]}
+                ],
+		supervisor:which_children(SubSysSup),
+                [ChSup]),
+    ?wait_match([{_,GroupPid,worker,[ssh_server_channel]}
+                ],
+                supervisor:which_children(ChSup),
+                [GroupPid]),
+
 
-    ?wait_match([{_, GroupPid,worker,[ssh_server_channel]}],
-		supervisor:which_children(ChannelSup),
-               [GroupPid]),
     {links,GroupLinks} = erlang:process_info(GroupPid, links),
-    [ShellPid] = GroupLinks--[ChannelSup],
+    ct:log("GroupPid = ~p, GroupLinks = ~p Sups0 = ~p",[GroupPid,GroupLinks,Sups0]),
+    [ShellPid] = GroupLinks--[ChSup],
     ct:log("GroupPid = ~p, ShellPid = ~p",[GroupPid,ShellPid]),
 
     receive
-        {ssh_cm,ConnectionRef, {data, ChannelId0, 0, <<"TimeoutShell started!\r\n">>}} ->
+        {ssh_cm,ConnectionRef, {data, ChannelId0, 0, <<"TimeoutShell started!",Rest/binary>>}} ->
+            ct:log("TimeoutShell started. Rest = ~p", [Rest]),
             receive
                 %%---- wait for the subsystem to terminate
                 {ssh_cm,ConnectionRef,{closed,ChannelId0}} ->
@@ -336,123 +347,147 @@ shell_channel_tree(Config) ->
                             ct:fail("Sup tree changed!")
                     end
             after 10000 ->
+                    ct:log("~p:~p  Flush unexpected: ~p", [?MODULE,?LINE,flush_rest()]),
                     ssh:close(ConnectionRef),
                     ssh:stop_daemon(Daemon),
-                    ct:fail("CLI Timeout")
+                    ct:fail("CLI Timeout 1")
             end
     after 10000 ->
+            ct:log("~p:~p  Flush unexpected: ~p", [?MODULE,?LINE,flush_rest()]),
             ssh:close(ConnectionRef),
             ssh:stop_daemon(Daemon),
-            ct:fail("CLI Timeout")
+            ct:fail("CLI Timeout 2")
     end.
 
-
 chk_empty_con_daemon(Daemon) ->
-    ?wait_match([{_,SubSysSup, supervisor,[ssh_subsystem_sup]},
-		 {{ssh_acceptor_sup,_,_,_}, AccSup, supervisor,[ssh_acceptor_sup]}],
+    ?wait_match([?SUB_SYSTEM_SUP(SubSysSup),
+		 ?ACCEPTOR_SUP(AccSup,_)
+                ],
 		supervisor:which_children(Daemon),
                 [SubSysSup,AccSup]),
-    ?wait_match([{_,_, supervisor,
-                  [ssh_tcpip_forward_acceptor_sup]},
-                 {{server,ssh_connection_sup, _,_},
-		  ConnectionSup, supervisor,
-		  [ssh_connection_sup]},
-		 {{server,ssh_channel_sup,_ ,_},
-		  ChannelSup,supervisor,
-		  [ssh_channel_sup]}],
+    ?wait_match([{_,FwdAccSup, supervisor,[ssh_tcpip_forward_acceptor_sup]},
+                 {_,ChSup,supervisor,[ssh_channel_sup]},
+                 {connection,ServerConnPid,worker,[ssh_connection_handler]}                 
+                ],
 		supervisor:which_children(SubSysSup),
-		[ConnectionSup,ChannelSup]),
-    ?wait_match([{{ssh_acceptor_sup,_,_,_},_,worker,[ssh_acceptor]}],
-		supervisor:which_children(AccSup)),
-    ?wait_match([{_, _, worker,[ssh_connection_handler]}],
-		supervisor:which_children(ConnectionSup)),
-    ?wait_match([], supervisor:which_children(ChannelSup)),
-    [ChannelSup, ConnectionSup, SubSysSup, AccSup].
+		[ChSup,FwdAccSup,ServerConnPid]),
+    ?wait_match([], supervisor:which_children(FwdAccSup)),
+    ?wait_match([], supervisor:which_children(ChSup)),
+    ?wait_match([?ACCEPTOR_WORKER(_,_)],
+                supervisor:which_children(AccSup),
+                []),
+    [SubSysSup, ChSup, ServerConnPid, AccSup, FwdAccSup].
 
 %%-------------------------------------------------------------------------
 %% Help functions
 %%-------------------------------------------------------------------------
 check_sshd_system_tree(Daemon, Host, Port, Config) -> 
     UserDir = proplists:get_value(userdir, Config),
-    {ok, Client} = ssh:connect(Host, Port, [{silently_accept_hosts, true},
-                                            {user_interaction, false},
-                                            {user, ?USER},
-                                            {password, ?PASSWD},
-                                            {user_dir, UserDir}]),
+    {ok, ClientConn} = ssh:connect(Host, Port, [{silently_accept_hosts, true},
+                                                {user_interaction, false},
+                                                {user, ?USER},
+                                                {password, ?PASSWD},
+                                                {user_dir, UserDir}]),
     
-    ?wait_match([{_,SubSysSup, supervisor,[ssh_subsystem_sup]},
-		 {{ssh_acceptor_sup,_,_,_}, AccSup, supervisor,[ssh_acceptor_sup]}],
+    ?wait_match([?SUB_SYSTEM_SUP(SubSysSup),
+		 ?ACCEPTOR_SUP(AccSup,_)
+                ],
 		supervisor:which_children(Daemon),
                 [SubSysSup,AccSup]),
     
-    ?wait_match([{_,
-                  _AcceptorSup, supervisor,
-                  [ssh_tcpip_forward_acceptor_sup]},
-                 {{server,ssh_connection_sup, _,_},
-		  ConnectionSup, supervisor,
-		  [ssh_connection_sup]},
-		 {{server,ssh_channel_sup,_ ,_},
-		  ChannelSup,supervisor,
-		  [ssh_channel_sup]}],
+    ?wait_match([{_,FwdAccSup, supervisor,[ssh_tcpip_forward_acceptor_sup]},
+                 {_,_,supervisor,[ssh_channel_sup]},
+                 {connection,ServerConn,worker,[ssh_connection_handler]}
+                ],
 		supervisor:which_children(SubSysSup),
-		[ConnectionSup,ChannelSup]),
-    
-    ?wait_match([{{ssh_acceptor_sup,_,_,_},_,worker,[ssh_acceptor]}],
+		[FwdAccSup,ServerConn]),
+    ?wait_match([], supervisor:which_children(FwdAccSup)),
+
+    ?wait_match([?ACCEPTOR_WORKER(_,_)],
 		supervisor:which_children(AccSup)),
     
-    ?wait_match([{_, _, worker,[ssh_connection_handler]}],
-		supervisor:which_children(ConnectionSup)),
     
-    ?wait_match([], supervisor:which_children(ChannelSup)),
-    
-    {ok,PidC} = ssh_sftp:start_channel(Client),
+    {ok,PidC} = ssh_sftp:start_channel(ClientConn),
+    ?wait_match([{_,FwdAccSup, supervisor,[ssh_tcpip_forward_acceptor_sup]},
+                 {_,ChSup,supervisor,[ssh_channel_sup]},
+                 {connection,ServerConn,worker,[ssh_connection_handler]}
+                ],
+		supervisor:which_children(SubSysSup),
+		[ChSup,ServerConn]),
 
-    ?wait_match([{_, PidS,worker,[ssh_server_channel]}],
-		supervisor:which_children(ChannelSup),
+    ?wait_match([{_,PidS,worker,[ssh_server_channel]}],
+                supervisor:which_children(ChSup),
                 [PidS]),
     true = (PidS =/= PidC),
+    ?wait_match([], supervisor:which_children(FwdAccSup)),
 
-    ssh:close(Client).
+    ssh:close(ClientConn).
 
 
-check_sshc_system_tree(SysSup, Connection, LocalIP, LocalPort, _Config) ->
-    ?wait_match([{_,SubSysSup,supervisor,[ssh_subsystem_sup]}],
+check_sshc_system_tree(SysSup, Connection, _LocalIP, _LocalPort, _Config) ->
+    ?wait_match([?SUB_SYSTEM_SUP(SubSysSup)],
                 supervisor:which_children(SysSup),
                 [SubSysSup]),
-    ?wait_match([{_,FwdAccSup, supervisor,
-                  [ssh_tcpip_forward_acceptor_sup]},
-                 {{client,ssh_connection_sup, LocalIP, LocalPort},
-		  ConnectionSup, supervisor,
-		  [ssh_connection_sup]},
-		 {{client,ssh_channel_sup, LocalIP, LocalPort},
-		  ChannelSup,supervisor,
-		  [ssh_channel_sup]}],
+    ?wait_match([{_,FwdAccSup, supervisor,[ssh_tcpip_forward_acceptor_sup]},
+                 {_,_,supervisor,[ssh_channel_sup]},
+                 {connection,Connection,worker,[ssh_connection_handler]}
+                ],
 		supervisor:which_children(SubSysSup),
-		[ConnectionSup,ChannelSup,FwdAccSup]),
-    ?wait_match([{_, Connection, worker,[ssh_connection_handler]}],
-		supervisor:which_children(ConnectionSup)),
-    ?wait_match([], supervisor:which_children(ChannelSup)),
+		[FwdAccSup]),
     ?wait_match([], supervisor:which_children(FwdAccSup)),
 
     {ok,ChPid1} = ssh_sftp:start_channel(Connection),
-    ?wait_match([{_,ChPid1,worker,[ssh_client_channel]}],
-                supervisor:which_children(ChannelSup)),
+    ?wait_match([{_,FwdAccSup, supervisor,[ssh_tcpip_forward_acceptor_sup]},
+                 {_,ChSup,supervisor, [ssh_channel_sup]},
+                 {connection,Connection,worker,[ssh_connection_handler]}
+                ],
+		supervisor:which_children(SubSysSup),
+		[ChSup,FwdAccSup]),
+    
+    ?wait_match([{_,ChPid1,worker,[ssh_client_channel]}
+                ],
+                supervisor:which_children(ChSup),
+                [ChPid1]),
 
     {ok,ChPid2} = ssh_sftp:start_channel(Connection),
-    ?wait_match([{_,ChPidA,worker,[ssh_client_channel]},
-                 {_,ChPidB,worker,[ssh_client_channel]}],
-                supervisor:which_children(ChannelSup),
-               [ChPidA, ChPidB]),
-    true = (lists:sort([ChPidA, ChPidB]) == lists:sort([ChPid1, ChPid2])),
+    ?wait_match([{_,FwdAccSup, supervisor,[ssh_tcpip_forward_acceptor_sup]},
+                 {_,ChSup,supervisor, [ssh_channel_sup]},
+                 {connection,Connection,worker,[ssh_connection_handler]}
+                ],
+		supervisor:which_children(SubSysSup),
+		[ChSup,FwdAccSup]),
+
+    ?wait_match([{_,ChPid2,worker,[ssh_client_channel]},
+                 {_,ChPid1,worker,[ssh_client_channel]}
+                ],
+                supervisor:which_children(ChSup),
+                [ChPid1,ChPid2]),
 
     ct:pal("Expect a SUPERVISOR REPORT with offender {pid,~p}....~n", [ChPid1]),
     exit(ChPid1, kill),
-    ?wait_match([{_,ChPid2,worker,[ssh_client_channel]}],
-                supervisor:which_children(ChannelSup)),
+    ?wait_match([{_,FwdAccSup, supervisor,[ssh_tcpip_forward_acceptor_sup]},
+                 {_,ChSup,supervisor, [ssh_channel_sup]},
+                 {connection,Connection,worker,[ssh_connection_handler]}
+                ],
+		supervisor:which_children(SubSysSup),
+		[ChSup,FwdAccSup]),
+
+    ?wait_match([{_,ChPid2,worker,[ssh_client_channel]}
+                ],
+                supervisor:which_children(ChSup),
+                [ChPid2]),
 
     ct:pal("Expect a SUPERVISOR REPORT with offender {pid,~p}....~n", [ChPid2]),
     exit(ChPid2, kill),
-    ?wait_match([], supervisor:which_children(ChannelSup)),
+    ?wait_match([{_,FwdAccSup, supervisor,[ssh_tcpip_forward_acceptor_sup]},
+                 {_,ChSup,supervisor, [ssh_channel_sup]},
+                 {connection,Connection,worker,[ssh_connection_handler]}
+                ],
+		supervisor:which_children(SubSysSup),
+		[ChSup,FwdAccSup]),
+
+    ?wait_match([], supervisor:which_children(ChSup)),
+
     ct:pal("... now there should not be any SUPERVISOR REPORT.~n", []).
 
 
@@ -463,21 +498,17 @@ acceptor_pid(DaemonPid) ->
                         Parent ! {self(), supsearch,
                                   [{AccPid,ListenAddr,Port}
 
-                                   || {{server,ssh_system_sup,ListenAddr,Port,NS},
-                                       DPid,supervisor,
-                                       [ssh_system_sup]} <- supervisor:which_children(sshd_sup),
+                                   || ?SYSTEM_SUP(DPid,#address{address=ListenAddr,port=Port,profile=NS})
+                                          <- supervisor:which_children(sshd_sup),
                                       DPid == DaemonPid,
 
-                                      {{ssh_acceptor_sup,L1,P1,NS1},
-                                       AccSupPid,supervisor,
-                                       [ssh_acceptor_sup]} <- supervisor:which_children(DaemonPid),
-                                      L1 == ListenAddr,
-                                      P1 == Port,
-                                      NS1 == NS1,
+                                      ?ACCEPTOR_SUP(AccSupPid,_)
+                                          <- supervisor:which_children(DaemonPid),
 
-                                      {{ssh_acceptor_sup,L2,P2,NS2},
-                                       AccPid,worker,
-                                       [ssh_acceptor]} <- supervisor:which_children(AccSupPid),
+                                      ?ACCEPTOR_WORKER(AccPid, #address{address=L2,
+                                                                        port=P2,
+                                                                        profile=NS2})
+                                          <- supervisor:which_children(AccSupPid),
                                       L2 == ListenAddr,
                                       P2 == Port,
                                       NS2 == NS]}
@@ -486,3 +517,10 @@ acceptor_pid(DaemonPid) ->
     after 2000 -> timeout
     end.
 
+%%%----------------------------------------------------------------
+flush_rest() -> lists:reverse(flush_rest([])).
+
+flush_rest(Acc) ->
+    receive Any -> [Any|Acc]
+    after 0 -> Acc
+    end.
