@@ -21,6 +21,8 @@
 %%
 -module(ssl_cert_SUITE).
 
+-behaviour(ct_suite).
+
 -include_lib("common_test/include/ct.hrl").
 -include_lib("public_key/include/public_key.hrl").
 
@@ -54,6 +56,8 @@
          client_auth_partial_chain_fun_fail/1,
          client_auth_sni/0,
          client_auth_sni/1,
+         client_auth_seelfsigned_peer/0,
+         client_auth_seelfsigned_peer/1,
          missing_root_cert_no_auth/0,
          missing_root_cert_no_auth/1,
          missing_root_cert_auth/0,
@@ -94,6 +98,8 @@
          unsupported_sign_algo_cert_client_auth/1,
          longer_chain/0,
          longer_chain/1,
+         duplicate_chain/0,
+         duplicate_chain/1,
          key_auth_ext_sign_only/0,
          key_auth_ext_sign_only/1,
          hello_retry_request/0,
@@ -141,7 +147,8 @@ groups() ->
      {rsa_pss_rsae_1_3, [], all_version_tests() ++ rsa_tests() ++ tls_1_3_tests() ++ tls_1_3_rsa_tests()},
      {rsa_pss_pss, [], all_version_tests() ++ rsa_tests()},
      {rsa_pss_pss_1_3, [], all_version_tests() ++ rsa_tests() ++ tls_1_3_tests() ++ tls_1_3_rsa_tests()},
-     {ecdsa_1_3, [], all_version_tests() ++ tls_1_3_tests()}
+     {ecdsa_1_3, [], all_version_tests() ++ tls_1_3_tests()},
+     {eddsa_1_3, [], all_version_tests() ++ tls_1_3_tests()}
     ].
 
 ssl_protocol_groups() ->
@@ -159,6 +166,7 @@ tls_1_2_protocol_groups() ->
 tls_1_3_protocol_groups() ->
     [{group, rsa_1_3},
      {group, ecdsa_1_3},
+     {group, eddsa_1_3},
      {group, rsa_pss_rsae_1_3},
      {group, rsa_pss_pss_1_3}
     ].
@@ -199,6 +207,7 @@ all_version_tests() ->
      client_auth_do_not_allow_partial_chain,
      client_auth_partial_chain_fun_fail,
      client_auth_sni,
+     client_auth_seelfsigned_peer,
      missing_root_cert_no_auth,
      missing_root_cert_auth,
      missing_root_cert_auth_user_verify_fun_accept,
@@ -222,7 +231,6 @@ init_per_suite(Config) ->
     catch crypto:stop(),
     try crypto:start() of
 	ok ->
-	    ssl_test_lib:clean_start(),
             Config
     catch _:_ ->
 	    {skip, "Crypto did not start"}
@@ -237,6 +245,7 @@ end_per_suite(_Config) ->
 init_per_group(GroupName, Config) ->
     case ssl_test_lib:is_protocol_version(GroupName) of
         true  ->
+            ssl_test_lib:clean_start(),
             ssl_test_lib:init_per_group(GroupName, 
                                         [{client_type, erlang},
                                          {server_type, erlang},
@@ -282,7 +291,7 @@ do_init_per_group(Alg, Config) when Alg == rsa_pss_rsae;
             {skip, "Missing EC crypto support"}
     end;
 do_init_per_group(Group, Config0) when Group == ecdsa;
-                                    Group == ecdsa_1_3 ->
+                                       Group == ecdsa_1_3 ->
 
     PKAlg = crypto:supports(public_keys),
     case lists:member(ecdsa, PKAlg) andalso (lists:member(ecdh, PKAlg) orelse lists:member(dh, PKAlg)) of
@@ -300,12 +309,36 @@ do_init_per_group(Group, Config0) when Group == ecdsa;
         false ->
             {skip, "Missing EC crypto support"}
     end;
+do_init_per_group(eddsa_1_3, Config0) ->
+    PKAlg = crypto:supports(public_keys),
+    PrivDir = proplists:get_value(priv_dir, Config0),
+    case lists:member(eddsa, PKAlg) andalso (lists:member(ecdh, PKAlg)) of
+        true ->
+            Conf = public_key:pkix_test_data(#{server_chain => #{root => ssl_test_lib:eddsa_conf(),
+                                                                 intermediates => [ssl_test_lib:eddsa_conf()],
+                                                                 peer =>  ssl_test_lib:eddsa_conf()},
+                                               client_chain => #{root => ssl_test_lib:eddsa_conf(),
+                                                                 intermediates => [ssl_test_lib:eddsa_conf()],
+                                                                 peer =>  ssl_test_lib:eddsa_conf()}}),
+            [{server_config, SOpts},
+             {client_config, COpts}] = x509_test:gen_pem_config_files(Conf, filename:join(PrivDir, "client_eddsa"),
+                                                                      filename:join(PrivDir, "server_eddsa")),
 
-do_init_per_group(Group, Config0) when Group == dsa ->
+            [{cert_key_alg, eddsa} |
+             lists:delete(cert_key_alg,
+                          [{client_cert_opts, COpts},
+                           {server_cert_opts, SOpts} |
+                           lists:delete(server_cert_opts,
+                                        lists:delete(client_cert_opts, Config0))]
+                         )];
+        false ->
+            {skip, "Missing EC crypto support"}
+    end;
+do_init_per_group(dsa, Config0) ->
     PKAlg = crypto:supports(public_keys),
     case lists:member(dss, PKAlg) andalso lists:member(dh, PKAlg) of
         true ->
-            Config = ssl_test_lib:make_dsa_cert(Config0),    
+            Config = ssl_test_lib:make_dsa_cert(Config0),
             COpts = proplists:get_value(client_dsa_opts, Config),
             SOpts = proplists:get_value(server_dsa_opts, Config),
             [{cert_key_alg, dsa} |
@@ -317,7 +350,7 @@ do_init_per_group(Group, Config0) when Group == dsa ->
         false ->
             {skip, "Missing DSS crypto support"}
     end;
-do_init_per_group(Group, Config) ->
+do_init_per_group(_Group, Config) ->
     Config.
 
 end_per_group(GroupName, Config) ->
@@ -382,6 +415,11 @@ client_auth_sni() ->
    ssl_cert_tests:client_auth_sni().
 client_auth_sni(Config) when is_list(Config) ->
     ssl_cert_tests:client_auth_sni(Config).
+%%--------------------------------------------------------------------
+client_auth_seelfsigned_peer() ->
+   ssl_cert_tests:client_auth_seelfsigned_peer().
+client_auth_seelfsigned_peer(Config) when is_list(Config) ->
+    ssl_cert_tests:client_auth_seelfsigned_peer(Config).
 
 %%--------------------------------------------------------------------
 missing_root_cert_no_auth() ->
@@ -479,7 +517,9 @@ missing_root_cert_auth_user_verify_fun_reject(Config) ->
 incomplete_chain_auth() ->
     [{doc,"Test that we can verify an incompleat chain when we have the certs to rebuild it"}].
 incomplete_chain_auth(Config) when is_list(Config) ->
-    DefaultCertConf = ssl_test_lib:default_cert_chain_conf(),
+    Prop = proplists:get_value(tc_group_properties, Config),
+    Group = proplists:get_value(name, Prop),
+    DefaultCertConf = ssl_test_lib:default_ecc_cert_chain_conf(Group),
     #{client_config := ClientOpts0,
       server_config := ServerOpts0} = ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
                                                                         [{server_chain, DefaultCertConf},
@@ -596,7 +636,8 @@ critical_extension_auth() ->
     [{doc,"Test cert that has a critical unknown extension in verify_peer mode"}].
 
 critical_extension_auth(Config) when is_list(Config) ->
-    DefaultCertConf = ssl_test_lib:default_cert_chain_conf(),
+    Prop = proplists:get_value(tc_group_properties, Config),
+    DefaultCertConf = ssl_test_lib:default_ecc_cert_chain_conf(proplists:get_value(name, Prop)),
     Ext = x509_test:extensions([{{2,16,840,1,113730,1,1}, <<3,2,6,192>>, true}]),
     #{client_config := ClientOpts0,
       server_config := ServerOpts0}  = ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
@@ -662,7 +703,8 @@ critical_extension_no_auth() ->
     [{doc,"Test cert that has a critical unknown extension in verify_none mode"}].
 
 critical_extension_no_auth(Config) when is_list(Config) ->
-    DefaultCertConf = ssl_test_lib:default_cert_chain_conf(),
+    Prop = proplists:get_value(tc_group_properties, Config),
+    DefaultCertConf = ssl_test_lib:default_ecc_cert_chain_conf(proplists:get_value(name, Prop)),
     Ext = x509_test:extensions([{{2,16,840,1,113730,1,1}, <<3,2,6,192>>, true}]),
     #{client_config := ClientOpts0,
       server_config := ServerOpts0}  = ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
@@ -680,7 +722,8 @@ extended_key_usage_auth() ->
     [{doc,"Test cert that has a critical extended_key_usage extension in server cert"}].
 
 extended_key_usage_auth(Config) when is_list(Config) -> 
-    DefaultCertConf = ssl_test_lib:default_cert_chain_conf(),
+    Prop = proplists:get_value(tc_group_properties, Config),
+    DefaultCertConf = ssl_test_lib:default_ecc_cert_chain_conf(proplists:get_value(name, Prop)),
     Ext = x509_test:extensions([{?'id-ce-extKeyUsage',
                                  [?'id-kp-serverAuth'], true}]),
     #{client_config := ClientOpts0,
@@ -750,7 +793,8 @@ cert_expired() ->
     [{doc,"Test server with expired certificate"}].
 
 cert_expired(Config) when is_list(Config) ->
-    DefaultCertConf = ssl_test_lib:default_cert_chain_conf(),
+    Prop = proplists:get_value(tc_group_properties, Config),
+    DefaultCertConf = ssl_test_lib:default_ecc_cert_chain_conf(proplists:get_value(name, Prop)),
     {Year, Month, Day} = date(),
     #{client_config := ClientOpts0,
       server_config := ServerOpts0} = ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
@@ -860,6 +904,62 @@ longer_chain(Config) when is_list(Config) ->
                                            proplists:delete(cacerts, ClientOpts0)], Config),
     ssl_test_lib:basic_test(ClientOpts, ServerOpts, Config).
 
+duplicate_chain() ->
+    [{doc, "Manual test of chain with duplicate entries"}].
+duplicate_chain(Config)
+  when is_list(Config) ->
+    Key1 = ssl_test_lib:hardcode_rsa_key(1),
+    Key2 = ssl_test_lib:hardcode_rsa_key(2),
+    Key3 = ssl_test_lib:hardcode_rsa_key(3),
+    Key4 = ssl_test_lib:hardcode_rsa_key(4),
+    Key5 = ssl_test_lib:hardcode_rsa_key(5),
+
+    #{server_config := ServerOpts0, client_config := ClientOpts0} =
+        public_key:pkix_test_data(#{server_chain => #{root => [{key, Key1}],
+                                                      peer => [{key, Key5}]},
+                                    client_chain => #{root => [{key, Key3}],
+                                                      intermediates => [[{key, Key2}], [{key, Key3}]],
+                                                      peer => [{key, Key1}]}}),
+
+    #{client_config := ClientOptsNew} =
+        public_key:pkix_test_data(#{server_chain => #{root => [{key, Key1}],
+                                                      peer => [{key, Key5}]},
+                                    client_chain => #{root => [{key, Key4}],
+                                                      intermediates => [[{key, Key2}], [{key, Key1}]],
+                                                      peer => [{key, Key1}]}}),
+
+    ServerCas0 = proplists:get_value(cacerts, ServerOpts0),
+    ClientCas0 = proplists:get_value(cacerts, ClientOpts0),
+
+    {[Peer,CI1,CI2,CROld], CROld} = chain_and_root(ClientOpts0),
+    {[_Peer,CI1New,CI2New,CRNew], CRNew} = chain_and_root(ClientOptsNew),
+
+    ServerCas = [CRNew|ServerCas0 -- [CROld]],
+    ServerOpts = ssl_test_lib:ssl_options([{verify, verify_peer} |
+                                           lists:keyreplace(cacerts, 1, ServerOpts0, {cacerts, ServerCas})],
+                                          Config),
+    ClientOpts = ssl_test_lib:ssl_options([{verify, verify_peer} |
+                                           lists:keyreplace(cacerts, 1,
+                                                            lists:keyreplace(cert, 1, ClientOpts0,
+                                                                             {cert, [Peer,CI1New,CI2New,CI1,CI2,CRNew,CROld]}),
+                                                            {cacerts, ClientCas0})],
+                                          Config),
+    ssl_test_lib:basic_test(ClientOpts, ServerOpts, Config),
+    ClientOpts2 = ssl_test_lib:ssl_options([{verify, verify_peer} |
+                                            lists:keyreplace(cacerts, 1,
+                                                             lists:keyreplace(cert, 1, ClientOpts0,
+                                                                              {cert, [Peer,CI1,CI1New,CI2,CI2New,CROld,CRNew]}),
+                                                             {cacerts, ClientCas0})],
+                                           Config),
+    ssl_test_lib:basic_test(ClientOpts2, ServerOpts, Config),
+    ok.
+
+chain_and_root(Config) ->
+    OwnCert = proplists:get_value(cert, Config),
+    {ok, ExtractedCAs} = ssl_pkix_db:extract_trusted_certs({der, proplists:get_value(cacerts, Config)}),
+    {ok, Root, Chain} = ssl_certificate:certificate_chain(OwnCert, ets:new(foo, []), ExtractedCAs, []),
+    {Chain, Root}.
+
 %%--------------------------------------------------------------------
 %% TLS 1.3 Test cases  -----------------------------------------------
 %%--------------------------------------------------------------------
