@@ -42,7 +42,8 @@ all() ->
      event_types, generic_timers, code_change,
      {group, sys},
      hibernate, auto_hibernate, enter_loop, {group, undef_callbacks},
-     undef_in_terminate, {group, format_log}].
+     undef_in_terminate, {group, format_log},
+     reply_by_alias_with_payload].
 
 groups() ->
     [{start, [], tcs(start)},
@@ -534,10 +535,10 @@ abnormal1(Config) ->
 	gen_statem:start(LocalSTM, ?MODULE, start_arg(Config, []), []),
 
     %% timeout call.
-    delayed = gen_statem:call(Name, {delayed_answer,1}, 100),
+    delayed = gen_statem:call(Name, {delayed_answer,100}, 2000),
     {timeout,_} =
 	?EXPECT_FAILURE(
-	   gen_statem:call(Name, {delayed_answer,1000}, 10),
+	   gen_statem:call(Name, {delayed_answer,2000}, 100),
 	   Reason),
     ok = gen_statem:stop(Name),
     ?t:sleep(1100),
@@ -582,8 +583,7 @@ abnormal1dirty(Config) ->
     ok = gen_statem:stop(Name),
     ?t:sleep(1100),
     case flush() of
-	[{Ref,delayed}] when is_reference(Ref) ->
-	    ok
+	[] -> ok
     end.
 
 %% Check that bad return values makes the stm crash. Note that we must
@@ -931,13 +931,11 @@ state_timeout(_Config) ->
 		       [{timeout,0,4},{state_timeout,0,5}]};
 		  (timeout, 4, {ok,3,Data}) ->
 		      %% Verify that timeout 0 is cancelled by
-		      %% enqueued state_timeout 0 and that
-		      %% multiple state_timeout 0 can be enqueued
+		      %% a state_timeout 0 event and that
+		      %% state_timeout 0 can be restarted
 		      {keep_state, {ok,4,Data},
 		       [{state_timeout,0,6},{timeout,0,7}]};
-		  (state_timeout, 5, {ok,4,Data}) ->
-		      {keep_state, {ok,5,Data}};
-		  (state_timeout, 6, {ok,5,{Time,From}}) ->
+		  (state_timeout, 6, {ok,4,{Time,From}}) ->
 		      {next_state, state3, 6,
 		       [{reply,From,ok},
 			{state_timeout,Time,8}]}
@@ -1254,7 +1252,7 @@ call_format_status(Config) ->
 	gen_statem:start(
 	  {local, gstm}, ?MODULE, start_arg(Config, []), []),
     Status2 = sys:get_status(gstm),
-    {status,Pid2,_Mod,[_PDict2,running,_,_,Data2]} = Status2,
+    {status,Pid2,Mod,[_PDict2,running,_,_,Data2]} = Status2,
     [format_status_called|_] = lists:reverse(Data2),
     stop_it(Pid2),
 
@@ -1265,7 +1263,7 @@ call_format_status(Config) ->
 	gen_statem:start(
 	  GlobalName1, ?MODULE, start_arg(Config, []), []),
     Status3 = sys:get_status(GlobalName1),
-    {status,Pid3,_Mod,[_PDict3,running,_,_,Data3]} = Status3,
+    {status,Pid3,Mod,[_PDict3,running,_,_,Data3]} = Status3,
     [format_status_called|_] = lists:reverse(Data3),
     stop_it(Pid3),
     GlobalName2 = {global,{name, "term"}},
@@ -1273,7 +1271,7 @@ call_format_status(Config) ->
 	gen_statem:start(
 	  GlobalName2, ?MODULE, start_arg(Config, []), []),
     Status4 = sys:get_status(GlobalName2),
-    {status,Pid4,_Mod,[_PDict4,running,_,_, Data4]} = Status4,
+    {status,Pid4,Mod,[_PDict4,running,_,_, Data4]} = Status4,
     [format_status_called|_] = lists:reverse(Data4),
     stop_it(Pid4),
 
@@ -1283,7 +1281,7 @@ call_format_status(Config) ->
     ViaName1 = {via,dummy_via,"CallFormatStatus"},
     {ok,Pid5} = gen_statem:start(ViaName1, ?MODULE, start_arg(Config, []), []),
     Status5 = sys:get_status(ViaName1),
-    {status,Pid5,_Mod, [_PDict5,running,_,_, Data5]} = Status5,
+    {status,Pid5,Mod, [_PDict5,running,_,_, Data5]} = Status5,
     [format_status_called|_] = lists:reverse(Data5),
     stop_it(Pid5),
     ViaName2 = {via,dummy_via,{name,"term"}},
@@ -1291,7 +1289,7 @@ call_format_status(Config) ->
 	gen_statem:start(
 	  ViaName2, ?MODULE, start_arg(Config, []), []),
     Status6 = sys:get_status(ViaName2),
-    {status,Pid6,_Mod,[_PDict6,running,_,_,Data6]} = Status6,
+    {status,Pid6,Mod,[_PDict6,running,_,_,Data6]} = Status6,
     [format_status_called|_] = lists:reverse(Data6),
     stop_it(Pid6).
 
@@ -1437,7 +1435,7 @@ hibernate(Config) ->
     {ok,Pid0} =
 	gen_statem:start_link(
 	  ?MODULE, start_arg(Config, hiber_now), []),
-    is_in_erlang_hibernate(Pid0),
+    wait_erlang_hibernate(Pid0),
     stop_it(Pid0),
     receive
 	{'EXIT',Pid0,normal} -> ok
@@ -1450,23 +1448,23 @@ hibernate(Config) ->
     true = ({current_function,{erlang,hibernate,3}} =/=
 		erlang:process_info(Pid,current_function)),
     hibernating = gen_statem:call(Pid, hibernate_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     good_morning = gen_statem:call(Pid, wakeup_sync),
     is_not_in_erlang_hibernate(Pid),
     hibernating = gen_statem:call(Pid, hibernate_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     please_just_five_more = gen_statem:call(Pid, snooze_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     good_morning = gen_statem:call(Pid, wakeup_sync),
     is_not_in_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, hibernate_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, wakeup_async),
     is_not_in_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, hibernate_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, snooze_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, wakeup_async),
     is_not_in_erlang_hibernate(Pid),
 
@@ -1474,14 +1472,14 @@ hibernate(Config) ->
     true =
 	({current_function,{erlang,hibernate,3}} =/=
 	     erlang:process_info(Pid, current_function)),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
 
     'alive!' = gen_statem:call(Pid, 'alive?'),
     true =
 	({current_function,{erlang,hibernate,3}} =/=
 	     erlang:process_info(Pid, current_function)),
     Pid ! hibernate_now,
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
 
     'alive!' = gen_statem:call(Pid, 'alive?'),
     true =
@@ -1489,34 +1487,34 @@ hibernate(Config) ->
 	     erlang:process_info(Pid, current_function)),
 
     hibernating = gen_statem:call(Pid, hibernate_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     good_morning = gen_statem:call(Pid, wakeup_sync),
     is_not_in_erlang_hibernate(Pid),
     hibernating = gen_statem:call(Pid, hibernate_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     please_just_five_more = gen_statem:call(Pid, snooze_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     good_morning = gen_statem:call(Pid, wakeup_sync),
     is_not_in_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, hibernate_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok  = gen_statem:cast(Pid, wakeup_async),
     is_not_in_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, hibernate_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, snooze_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, wakeup_async),
     is_not_in_erlang_hibernate(Pid),
 
     hibernating = gen_statem:call(Pid, hibernate_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     sys:suspend(Pid),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     sys:resume(Pid),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     receive after 1000 -> ok end,
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
 
     good_morning  = gen_statem:call(Pid, wakeup_sync),
     is_not_in_erlang_hibernate(Pid),
@@ -1532,15 +1530,16 @@ hibernate(Config) ->
 %% Auto-hibernation timeout
 auto_hibernate(Config) ->
     OldFl = process_flag(trap_exit, true),
-    HibernateAfterTimeout = 100,
+    HibernateAfterTimeout = 1000,
 
     {ok,Pid} =
         gen_statem:start_link(
-            ?MODULE, start_arg(Config, []), [{hibernate_after, HibernateAfterTimeout}]),
+            ?MODULE, start_arg(Config, []),
+          [{hibernate_after, HibernateAfterTimeout}]),
     %% After init test
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     %% After info test
     Pid ! {hping, self()},
     receive
@@ -1551,7 +1550,7 @@ auto_hibernate(Config) ->
     end,
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     %% After cast test
     ok = gen_statem:cast(Pid, {hping, self()}),
     receive
@@ -1562,42 +1561,42 @@ auto_hibernate(Config) ->
     end,
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     %% After call test
     hpong = gen_statem:call(Pid, hping),
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     %% Timer test 1
-    TimerTimeout1 = 50,
-    ok = gen_statem:call(Pid, {arm_htimer, self(), TimerTimeout1}),
+    TimerTimeout1 = HibernateAfterTimeout div 2,
+    ok = gen_statem:call(Pid, {start_htimer, self(), TimerTimeout1}),
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(TimerTimeout1),
     is_not_in_erlang_hibernate(Pid),
     receive
-        {Pid, htimer_armed} ->
+        {Pid, htimer_timeout} ->
             ok
     after 1000 ->
         ct:fail(timer1)
     end,
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     %% Timer test 2
-    TimerTimeout2 = 150,
-    ok = gen_statem:call(Pid, {arm_htimer, self(), TimerTimeout2}),
+    TimerTimeout2 = HibernateAfterTimeout * 2,
+    ok = gen_statem:call(Pid, {start_htimer, self(), TimerTimeout2}),
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     receive
-        {Pid, htimer_armed} ->
+        {Pid, htimer_timeout} ->
             ok
-    after 1000 ->
+    after TimerTimeout2 ->
         ct:fail(timer2)
     end,
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     stop_it(Pid),
     process_flag(trap_exit, OldFl),
     receive
@@ -1607,38 +1606,38 @@ auto_hibernate(Config) ->
     end,
     ok = verify_empty_msgq().
 
-is_in_erlang_hibernate(Pid) ->
-    receive after 1 -> ok end,
-    is_in_erlang_hibernate_1(200, Pid).
 
-is_in_erlang_hibernate_1(0, Pid) ->
+wait_erlang_hibernate(Pid) ->
+    receive after 1 -> ok end,
+    wait_erlang_hibernate_1(200, Pid).
+
+wait_erlang_hibernate_1(0, Pid) ->
     ct:log("~p\n", [erlang:process_info(Pid, current_function)]),
-    ct:fail(not_in_erlang_hibernate_3);
-is_in_erlang_hibernate_1(N, Pid) ->
+    ct:fail(should_be_in_erlang_hibernate_3);
+wait_erlang_hibernate_1(N, Pid) ->
     {current_function,MFA} = erlang:process_info(Pid, current_function),
     case MFA of
 	{erlang,hibernate,3} ->
 	    ok;
 	_ ->
 	    receive after 10 -> ok end,
-	    is_in_erlang_hibernate_1(N-1, Pid)
+	    wait_erlang_hibernate_1(N-1, Pid)
     end.
 
 is_not_in_erlang_hibernate(Pid) ->
     receive after 1 -> ok end,
     is_not_in_erlang_hibernate_1(200, Pid).
 
-is_not_in_erlang_hibernate_1(0, Pid) ->
-    ct:log("~p\n", [erlang:process_info(Pid, current_function)]),
-    ct:fail(not_in_erlang_hibernate_3);
+is_not_in_erlang_hibernate_1(0, _Pid) ->
+    ct:fail(should_not_be_in_erlang_hibernate_3);
 is_not_in_erlang_hibernate_1(N, Pid) ->
     {current_function,MFA} = erlang:process_info(Pid, current_function),
     case MFA of
-	{erlang,hibernate,3} ->
+ 	{erlang,hibernate,3} ->
 	    receive after 10 -> ok end,
 	    is_not_in_erlang_hibernate_1(N-1, Pid);
-	_ ->
-	    ok
+ 	_ ->
+ 	    ok
     end.
 
 
@@ -2213,6 +2212,23 @@ stacktrace() ->
 flatten_format_log(Report, Format) ->
     lists:flatten(gen_statem:format_log(Report, Format)).
 
+reply_by_alias_with_payload(Config) when is_list(Config) ->
+    %% "Payload" version of tag not used yet, but make sure
+    %% gen_statem:reply/2 works with it...
+    %%
+    %% Whitebox...
+    Reply = make_ref(),
+    Alias = alias(),
+    Tag = [[alias|Alias], "payload"],
+    spawn_link(fun () ->
+                       gen_statem:reply({undefined, Tag},
+                                        Reply)
+               end),
+    receive
+        {[[alias|Alias]|_] = Tag, Reply} ->
+            ok
+    end.
+
 %%
 %% Functionality check
 %%
@@ -2430,10 +2446,10 @@ idle(cast, {hping,Pid}, Data) ->
     {keep_state, Data};
 idle({call, From}, hping, _Data) ->
     {keep_state_and_data, [{reply, From, hpong}]};
-idle({call, From}, {arm_htimer, Pid, Timeout}, _Data) ->
-    {keep_state_and_data, [{reply, From, ok}, {timeout, Timeout, {arm_htimer, Pid}}]};
-idle(timeout, {arm_htimer, Pid}, _Data) ->
-    Pid ! {self(), htimer_armed},
+idle({call, From}, {start_htimer, Pid, Timeout}, _Data) ->
+    {keep_state_and_data, [{reply, From, ok}, {timeout, Timeout, {htimer, Pid}}]};
+idle(timeout, {htimer, Pid}, _Data) ->
+    Pid ! {self(), htimer_timeout},
     keep_state_and_data;
 idle(cast, {connect,Pid}, Data) ->
     Pid ! accept,
