@@ -197,7 +197,7 @@ name_it(intersection)  -> ' * ';
 name_it(difference)    -> ' - ';
 name_it(Name) -> Name.   
 
--file("/net/isildur/ldisk/daily_build/23_prebuild_opu_o.2020-12-15_21/otp_src_23/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
+-file("/daily_build/otp_src/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
 %%
 %% %CopyrightBegin%
 %%
@@ -225,15 +225,15 @@ name_it(Name) -> Name.
 
 -spec parse(Tokens :: list()) -> yecc_ret().
 parse(Tokens) ->
-    yeccpars0(Tokens, {no_func, no_line}, 0, [], []).
+    yeccpars0(Tokens, {no_func, no_location}, 0, [], []).
 
 -spec parse_and_scan({function() | {atom(), atom()}, [_]}
                      | {atom(), atom(), [_]}) -> yecc_ret().
 parse_and_scan({F, A}) ->
-    yeccpars0([], {{F, A}, no_line}, 0, [], []);
+    yeccpars0([], {{F, A}, no_location}, 0, [], []);
 parse_and_scan({M, F, A}) ->
     Arity = length(A),
-    yeccpars0([], {{fun M:F/Arity, A}, no_line}, 0, [], []).
+    yeccpars0([], {{fun M:F/Arity, A}, no_location}, 0, [], []).
 
 -spec format_error(any()) -> [char() | list()].
 format_error(Message) ->
@@ -247,9 +247,9 @@ format_error(Message) ->
 %% To be used in grammar files to throw an error message to the parser
 %% toplevel. Doesn't have to be exported!
 -compile({nowarn_unused_function, return_error/2}).
--spec return_error(integer(), any()) -> no_return().
-return_error(Line, Message) ->
-    throw({error, {Line, ?MODULE, Message}}).
+-spec return_error(erl_anno:location(), any()) -> no_return().
+return_error(Location, Message) ->
+    throw({error, {Location, ?MODULE, Message}}).
 
 -define(CODE_VERSION, "1.4").
 
@@ -264,7 +264,7 @@ yeccpars0(Tokens, Tzr, State, States, Vstack) ->
             catch _:_ -> erlang:raise(error, Error, Stacktrace)
             end;
         %% Probably thrown from return_error/2:
-        throw: {error, {_Line, ?MODULE, _M}} = Error ->
+        throw: {error, {_Location, ?MODULE, _M}} = Error ->
             Error
     end.
 
@@ -281,22 +281,22 @@ yecc_error_type(function_clause, [{?MODULE,F,ArityOrArgs,_} | _]) ->
 
 yeccpars1([Token | Tokens], Tzr, State, States, Vstack) ->
     yeccpars2(State, element(1, Token), States, Vstack, Token, Tokens, Tzr);
-yeccpars1([], {{F, A},_Line}, State, States, Vstack) ->
+yeccpars1([], {{F, A},_Location}, State, States, Vstack) ->
     case apply(F, A) of
-        {ok, Tokens, Endline} ->
-            yeccpars1(Tokens, {{F, A}, Endline}, State, States, Vstack);
-        {eof, Endline} ->
-            yeccpars1([], {no_func, Endline}, State, States, Vstack);
-        {error, Descriptor, _Endline} ->
+        {ok, Tokens, EndLocation} ->
+            yeccpars1(Tokens, {{F, A}, EndLocation}, State, States, Vstack);
+        {eof, EndLocation} ->
+            yeccpars1([], {no_func, EndLocation}, State, States, Vstack);
+        {error, Descriptor, _EndLocation} ->
             {error, Descriptor}
     end;
-yeccpars1([], {no_func, no_line}, State, States, Vstack) ->
+yeccpars1([], {no_func, no_location}, State, States, Vstack) ->
     Line = 999999,
     yeccpars2(State, '$end', States, Vstack, yecc_end(Line), [],
               {no_func, Line});
-yeccpars1([], {no_func, Endline}, State, States, Vstack) ->
-    yeccpars2(State, '$end', States, Vstack, yecc_end(Endline), [],
-              {no_func, Endline}).
+yeccpars1([], {no_func, EndLocation}, State, States, Vstack) ->
+    yeccpars2(State, '$end', States, Vstack, yecc_end(EndLocation), [],
+              {no_func, EndLocation}).
 
 %% yeccpars1/7 is called from generated code.
 %%
@@ -307,21 +307,19 @@ yeccpars1([], {no_func, Endline}, State, States, Vstack) ->
 yeccpars1(State1, State, States, Vstack, Token0, [Token | Tokens], Tzr) ->
     yeccpars2(State, element(1, Token), [State1 | States],
               [Token0 | Vstack], Token, Tokens, Tzr);
-yeccpars1(State1, State, States, Vstack, Token0, [], {{_F,_A}, _Line}=Tzr) ->
+yeccpars1(State1, State, States, Vstack, Token0, [], {{_F,_A}, _Location}=Tzr) ->
     yeccpars1([], Tzr, State, [State1 | States], [Token0 | Vstack]);
-yeccpars1(State1, State, States, Vstack, Token0, [], {no_func, no_line}) ->
-    Line = yecctoken_end_location(Token0),
+yeccpars1(State1, State, States, Vstack, Token0, [], {no_func, no_location}) ->
+    Location = yecctoken_end_location(Token0),
     yeccpars2(State, '$end', [State1 | States], [Token0 | Vstack],
-              yecc_end(Line), [], {no_func, Line});
-yeccpars1(State1, State, States, Vstack, Token0, [], {no_func, Line}) ->
+              yecc_end(Location), [], {no_func, Location});
+yeccpars1(State1, State, States, Vstack, Token0, [], {no_func, Location}) ->
     yeccpars2(State, '$end', [State1 | States], [Token0 | Vstack],
-              yecc_end(Line), [], {no_func, Line}).
+              yecc_end(Location), [], {no_func, Location}).
 
 %% For internal use only.
-yecc_end({Line,_Column}) ->
-    {'$end', Line};
-yecc_end(Line) ->
-    {'$end', Line}.
+yecc_end(Location) ->
+    {'$end', Location}.
 
 yecctoken_end_location(Token) ->
     try erl_anno:end_location(element(2, Token)) of
@@ -369,7 +367,7 @@ yecctoken2string(Other) ->
 
 
 
--file("xref_parser.erl", 372).
+-file("xref_parser.erl", 370).
 
 -dialyzer({nowarn_function, yeccpars2/7}).
 yeccpars2(0=S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -599,7 +597,8 @@ yeccpars2_2(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccgoto_regvar(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 yeccpars2_3(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_xref(hd(Ss), Cat, Ss, Stack, T, Ts, Tzr).
+ NewStack = yeccpars2_3_(Stack),
+ yeccgoto_xref(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 yeccpars2_4(S, ',', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 74, Ss, Stack, T, Ts, Tzr);
@@ -608,7 +607,8 @@ yeccpars2_4(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccgoto_statements(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 yeccpars2_5(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_regatom(hd(Ss), Cat, Ss, Stack, T, Ts, Tzr).
+ NewStack = yeccpars2_5_(Stack),
+ yeccgoto_regatom(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 yeccpars2_6(S, decl, Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 33, Ss, Stack, T, Ts, Tzr);
@@ -643,10 +643,12 @@ yeccpars2_6(_S, '|||', Ss, Stack, T, Ts, Tzr) ->
  NewStack = 'yeccpars2_6_\'|||\''(Stack),
  yeccpars2_73(_S, '|||', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_regatom(hd(Ss), Cat, Ss, Stack, T, Ts, Tzr).
+ NewStack = yeccpars2_6_(Stack),
+ yeccgoto_regatom(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 yeccpars2_7(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_expr(hd(Ss), Cat, Ss, Stack, T, Ts, Tzr).
+ NewStack = yeccpars2_7_(Stack),
+ yeccgoto_expr(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 -dialyzer({nowarn_function, yeccpars2_8/7}).
 yeccpars2_8(S, ':', Ss, Stack, T, Ts, Tzr) ->
@@ -657,23 +659,32 @@ yeccpars2_8(_, _, _, _, T, _, _) ->
 %% yeccpars2_9: see yeccpars2_0
 
 yeccpars2_10(_S, '#', Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_prefix_op(hd(Ss), '#', Ss, Stack, T, Ts, Tzr);
+ NewStack = 'yeccpars2_10_\'#\''(Stack),
+ yeccgoto_prefix_op(hd(Ss), '#', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, '(', Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_prefix_op(hd(Ss), '(', Ss, Stack, T, Ts, Tzr);
+ NewStack = 'yeccpars2_10_\'(\''(Stack),
+ yeccgoto_prefix_op(hd(Ss), '(', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, '[', Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_prefix_op(hd(Ss), '[', Ss, Stack, T, Ts, Tzr);
+ NewStack = 'yeccpars2_10_\'[\''(Stack),
+ yeccgoto_prefix_op(hd(Ss), '[', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, atom, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_prefix_op(hd(Ss), atom, Ss, Stack, T, Ts, Tzr);
+ NewStack = yeccpars2_10_atom(Stack),
+ yeccgoto_prefix_op(hd(Ss), atom, Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, edge, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_prefix_op(hd(Ss), edge, Ss, Stack, T, Ts, Tzr);
+ NewStack = yeccpars2_10_edge(Stack),
+ yeccgoto_prefix_op(hd(Ss), edge, Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, string, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_prefix_op(hd(Ss), string, Ss, Stack, T, Ts, Tzr);
+ NewStack = yeccpars2_10_string(Stack),
+ yeccgoto_prefix_op(hd(Ss), string, Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, var, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_prefix_op(hd(Ss), var, Ss, Stack, T, Ts, Tzr);
+ NewStack = yeccpars2_10_var(Stack),
+ yeccgoto_prefix_op(hd(Ss), var, Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, vertex, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_prefix_op(hd(Ss), vertex, Ss, Stack, T, Ts, Tzr);
+ NewStack = yeccpars2_10_vertex(Stack),
+ yeccgoto_prefix_op(hd(Ss), vertex, Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, '{', Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_prefix_op(hd(Ss), '{', Ss, Stack, T, Ts, Tzr);
+ NewStack = 'yeccpars2_10_\'{\''(Stack),
+ yeccgoto_prefix_op(hd(Ss), '{', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, '$end', Ss, Stack, T, Ts, Tzr) ->
  NewStack = 'yeccpars2_10_\'$end\''(Stack),
  yeccgoto_const(hd(Ss), '$end', Ss, NewStack, T, Ts, Tzr);
@@ -738,7 +749,8 @@ yeccpars2_13(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_59(_S, Cat, [13 | Ss], NewStack, T, Ts, Tzr).
 
 yeccpars2_14(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_constant(hd(Ss), Cat, Ss, Stack, T, Ts, Tzr).
+ NewStack = yeccpars2_14_(Stack),
+ yeccgoto_constant(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 %% yeccpars2_15: see yeccpars2_0
 
@@ -1048,7 +1060,8 @@ yeccpars2_63(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccgoto_regvar(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 yeccpars2_64(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_regatom(hd(Ss), Cat, Ss, Stack, T, Ts, Tzr).
+ NewStack = yeccpars2_64_(Stack),
+ yeccgoto_regatom(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 -dialyzer({nowarn_function, yeccpars2_65/7}).
 yeccpars2_65(S, '/', Ss, Stack, T, Ts, Tzr) ->
@@ -1066,10 +1079,12 @@ yeccpars2_67(S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_cont_0(S, Cat, Ss, Stack, T, Ts, Tzr).
 
 yeccpars2_68(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_regint(hd(Ss), Cat, Ss, Stack, T, Ts, Tzr).
+ NewStack = yeccpars2_68_(Stack),
+ yeccgoto_regint(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 yeccpars2_69(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
- yeccgoto_regint(hd(Ss), Cat, Ss, Stack, T, Ts, Tzr).
+ NewStack = yeccpars2_69_(Stack),
+ yeccgoto_regint(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 yeccpars2_70(S, decl, Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 33, Ss, Stack, T, Ts, Tzr);
@@ -1584,688 +1599,825 @@ yeccgoto_xref(0, Cat, Ss, Stack, T, Ts, Tzr) ->
 -compile({inline,'yeccpars2_2_\'$end\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_2_\'$end\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_2_\'*\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_2_\'*\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_2_\'+\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_2_\'+\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_2_\'-\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_2_\'-\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_2_\'of\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_2_\'of\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_2_\'|\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_2_\'|\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_2_\'||\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_2_\'||\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_2_\'|||\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_2_\'|||\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,yeccpars2_2_/1}).
 -file("xref_parser.yrl", 96).
 yeccpars2_2_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   check_regexp_variable ( __1 )
+                     check_regexp_variable(___1)
+  end | __Stack].
+
+-compile({inline,yeccpars2_3_/1}).
+-file("xref_parser.yrl", 31).
+yeccpars2_3_(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                     ___1
   end | __Stack].
 
 -compile({inline,yeccpars2_4_/1}).
 -file("xref_parser.yrl", 54).
 yeccpars2_4_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   [ __1 ]
+                          [___1]
+  end | __Stack].
+
+-compile({inline,yeccpars2_5_/1}).
+-file("xref_parser.yrl", 89).
+yeccpars2_5_(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                    ___1
   end | __Stack].
 
 -compile({inline,'yeccpars2_6_\'$end\''/1}).
 -file("xref_parser.yrl", 102).
 'yeccpars2_6_\'$end\''(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,'yeccpars2_6_\')\''/1}).
 -file("xref_parser.yrl", 102).
 'yeccpars2_6_\')\''(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,'yeccpars2_6_\'*\''/1}).
 -file("xref_parser.yrl", 102).
 'yeccpars2_6_\'*\''(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,'yeccpars2_6_\'+\''/1}).
 -file("xref_parser.yrl", 102).
 'yeccpars2_6_\'+\''(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,'yeccpars2_6_\',\''/1}).
 -file("xref_parser.yrl", 102).
 'yeccpars2_6_\',\''(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,'yeccpars2_6_\'-\''/1}).
 -file("xref_parser.yrl", 102).
 'yeccpars2_6_\'-\''(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,'yeccpars2_6_\'of\''/1}).
 -file("xref_parser.yrl", 102).
 'yeccpars2_6_\'of\''(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,'yeccpars2_6_\'|\''/1}).
 -file("xref_parser.yrl", 102).
 'yeccpars2_6_\'|\''(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,'yeccpars2_6_\'||\''/1}).
 -file("xref_parser.yrl", 102).
 'yeccpars2_6_\'||\''(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,'yeccpars2_6_\'|||\''/1}).
 -file("xref_parser.yrl", 102).
 'yeccpars2_6_\'|||\''(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
+-compile({inline,yeccpars2_6_/1}).
+-file("xref_parser.yrl", 87).
+yeccpars2_6_(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                    ___1
+  end | __Stack].
+
+-compile({inline,yeccpars2_7_/1}).
+-file("xref_parser.yrl", 71).
+yeccpars2_7_(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                 ___1
+  end | __Stack].
+
+-compile({inline,'yeccpars2_10_\'#\''/1}).
+-file("xref_parser.yrl", 44).
+'yeccpars2_10_\'#\''(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                  ___1
+  end | __Stack].
+
+-compile({inline,'yeccpars2_10_\'(\''/1}).
+-file("xref_parser.yrl", 44).
+'yeccpars2_10_\'(\''(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                  ___1
+  end | __Stack].
+
+-compile({inline,'yeccpars2_10_\'[\''/1}).
+-file("xref_parser.yrl", 44).
+'yeccpars2_10_\'[\''(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                  ___1
+  end | __Stack].
+
+-compile({inline,yeccpars2_10_atom/1}).
+-file("xref_parser.yrl", 44).
+yeccpars2_10_atom(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                  ___1
+  end | __Stack].
+
+-compile({inline,yeccpars2_10_edge/1}).
+-file("xref_parser.yrl", 44).
+yeccpars2_10_edge(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                  ___1
+  end | __Stack].
+
+-compile({inline,yeccpars2_10_string/1}).
+-file("xref_parser.yrl", 44).
+yeccpars2_10_string(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                  ___1
+  end | __Stack].
+
+-compile({inline,yeccpars2_10_var/1}).
+-file("xref_parser.yrl", 44).
+yeccpars2_10_var(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                  ___1
+  end | __Stack].
+
+-compile({inline,yeccpars2_10_vertex/1}).
+-file("xref_parser.yrl", 44).
+yeccpars2_10_vertex(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                  ___1
+  end | __Stack].
+
+-compile({inline,'yeccpars2_10_\'{\''/1}).
+-file("xref_parser.yrl", 44).
+'yeccpars2_10_\'{\''(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                  ___1
+  end | __Stack].
+
 -compile({inline,'yeccpars2_10_\'$end\''/1}).
 -file("xref_parser.yrl", 79).
 'yeccpars2_10_\'$end\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_10_\')\''/1}).
 -file("xref_parser.yrl", 79).
 'yeccpars2_10_\')\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_10_\'*\''/1}).
 -file("xref_parser.yrl", 79).
 'yeccpars2_10_\'*\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_10_\'+\''/1}).
 -file("xref_parser.yrl", 79).
 'yeccpars2_10_\'+\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_10_\',\''/1}).
 -file("xref_parser.yrl", 79).
 'yeccpars2_10_\',\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_10_\'-\''/1}).
 -file("xref_parser.yrl", 79).
 'yeccpars2_10_\'-\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,yeccpars2_10_decl/1}).
 -file("xref_parser.yrl", 79).
 yeccpars2_10_decl(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_10_\'of\''/1}).
 -file("xref_parser.yrl", 79).
 'yeccpars2_10_\'of\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_10_\'|\''/1}).
 -file("xref_parser.yrl", 79).
 'yeccpars2_10_\'|\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_10_\'||\''/1}).
 -file("xref_parser.yrl", 79).
 'yeccpars2_10_\'||\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_10_\'|||\''/1}).
 -file("xref_parser.yrl", 79).
 'yeccpars2_10_\'|||\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,yeccpars2_10_/1}).
 -file("xref_parser.yrl", 88).
 yeccpars2_10_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { atom , __1 }
+                {atom, ___1}
   end | __Stack].
 
 -compile({inline,yeccpars2_11_/1}).
 -file("xref_parser.yrl", 55).
 yeccpars2_11_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   [ __1 ]
+                     [___1]
   end | __Stack].
 
 -compile({inline,yeccpars2_13_/1}).
 -file("xref_parser.yrl", 102).
 yeccpars2_13_(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
+-compile({inline,yeccpars2_14_/1}).
+-file("xref_parser.yrl", 77).
+yeccpars2_14_(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                    ___1
+  end | __Stack].
+
 -compile({inline,yeccpars2_16_/1}).
 -file("xref_parser.yrl", 38).
 yeccpars2_16_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   '#'
+                   '#'
   end | __Stack].
 
 -compile({inline,yeccpars2_19_/1}).
 -file("xref_parser.yrl", 98).
 yeccpars2_19_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   value_of ( __1 )
+                      value_of(___1)
   end | __Stack].
 
 -compile({inline,yeccpars2_20_/1}).
 -file("xref_parser.yrl", 80).
 yeccpars2_20_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   value_of ( __1 )
+                value_of(___1)
   end | __Stack].
 
 -compile({inline,yeccpars2_21_/1}).
 -file("xref_parser.yrl", 95).
 yeccpars2_21_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   check_regexp ( value_of ( __1 ) )
+                   check_regexp(value_of(___1))
   end | __Stack].
 
 -compile({inline,yeccpars2_22_/1}).
 -file("xref_parser.yrl", 99).
 yeccpars2_22_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   value_of ( __1 )
+                      value_of(___1)
   end | __Stack].
 
 -compile({inline,yeccpars2_23_/1}).
 -file("xref_parser.yrl", 81).
 yeccpars2_23_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   value_of ( __1 )
+                  value_of(___1)
   end | __Stack].
 
 -compile({inline,yeccpars2_25_/1}).
 -file("xref_parser.yrl", 79).
 yeccpars2_25_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { constant , unknown , vertex , __1 }
+              {constant, unknown, vertex, ___1}
   end | __Stack].
 
 -compile({inline,yeccpars2_26_/1}).
 -file("xref_parser.yrl", 74).
 yeccpars2_26_(__Stack0) ->
  [begin
-   [ ]
+                        []
   end | __Stack0].
 
 -compile({inline,yeccpars2_29_/1}).
 -file("xref_parser.yrl", 74).
 yeccpars2_29_(__Stack0) ->
  [begin
-   [ ]
+                        []
   end | __Stack0].
 
 -compile({inline,yeccpars2_30_/1}).
 -file("xref_parser.yrl", 75).
 yeccpars2_30_(__Stack0) ->
- [__3,__2,__1 | __Stack] = __Stack0,
+ [___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   [ __2 | __3 ]
+                                      [___2 | ___3]
   end | __Stack].
 
 -compile({inline,yeccpars2_31_/1}).
 -file("xref_parser.yrl", 102).
 yeccpars2_31_(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,yeccpars2_32_/1}).
 -file("xref_parser.yrl", 61).
 yeccpars2_32_(__Stack0) ->
- [__5,__4,__3,__2,__1 | __Stack] = __Stack0,
+ [___5,___4,___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   type ( { tuple , [ __2 | __3 ] } , __5 )
+                                          type({tuple, [___2 | ___3]}, ___5)
   end | __Stack].
 
 -compile({inline,yeccpars2_33_/1}).
 -file("xref_parser.yrl", 101).
 yeccpars2_33_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   value_of ( __1 )
+                       value_of(___1)
   end | __Stack].
 
 -compile({inline,yeccpars2_34_/1}).
 -file("xref_parser.yrl", 74).
 yeccpars2_34_(__Stack0) ->
  [begin
-   [ ]
+                        []
   end | __Stack0].
 
 -compile({inline,yeccpars2_36_/1}).
 -file("xref_parser.yrl", 102).
 yeccpars2_36_(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,yeccpars2_37_/1}).
 -file("xref_parser.yrl", 60).
 yeccpars2_37_(__Stack0) ->
- [__5,__4,__3,__2,__1 | __Stack] = __Stack0,
+ [___5,___4,___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   type ( { list , [ __2 | __3 ] } , __5 )
+                                          type({list, [___2 | ___3]}, ___5)
   end | __Stack].
 
 -compile({inline,'yeccpars2_38_\'$end\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_38_\'$end\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_38_\')\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_38_\')\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_38_\'*\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_38_\'*\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_38_\'+\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_38_\'+\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_38_\',\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_38_\',\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_38_\'-\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_38_\'-\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_38_\'of\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_38_\'of\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_38_\'|\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_38_\'|\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_38_\'||\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_38_\'||\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,'yeccpars2_38_\'|||\''/1}).
 -file("xref_parser.yrl", 63).
 'yeccpars2_38_\'|||\''(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { variable , __1 }
+                   {variable, ___1}
   end | __Stack].
 
 -compile({inline,yeccpars2_38_/1}).
 -file("xref_parser.yrl", 96).
 yeccpars2_38_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   check_regexp_variable ( __1 )
+                     check_regexp_variable(___1)
   end | __Stack].
 
 -compile({inline,yeccpars2_41_/1}).
 -file("xref_parser.yrl", 43).
 yeccpars2_41_(__Stack0) ->
- [__3,__2,__1 | __Stack] = __Stack0,
+ [___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   value_of ( __2 )
+                            value_of(___2)
   end | __Stack].
 
 -compile({inline,yeccpars2_46_/1}).
 -file("xref_parser.yrl", 72).
 yeccpars2_46_(__Stack0) ->
- [__3,__2,__1 | __Stack] = __Stack0,
+ [___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   __2
+                       ___2
   end | __Stack].
 
 -compile({inline,yeccpars2_47_/1}).
 -file("xref_parser.yrl", 37).
 yeccpars2_47_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   intersection
+                   intersection
   end | __Stack].
 
 -compile({inline,yeccpars2_48_/1}).
 -file("xref_parser.yrl", 35).
 yeccpars2_48_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   union
+                   union
   end | __Stack].
 
 -compile({inline,yeccpars2_49_/1}).
 -file("xref_parser.yrl", 36).
 yeccpars2_49_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   difference
+                   difference
   end | __Stack].
 
 -compile({inline,yeccpars2_50_/1}).
 -file("xref_parser.yrl", 42).
 yeccpars2_50_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   'of'
+                    'of'
   end | __Stack].
 
 -compile({inline,yeccpars2_51_/1}).
 -file("xref_parser.yrl", 39).
 yeccpars2_51_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   '|'
+                   '|'
   end | __Stack].
 
 -compile({inline,yeccpars2_52_/1}).
 -file("xref_parser.yrl", 40).
 yeccpars2_52_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   '||'
+                    '||'
   end | __Stack].
 
 -compile({inline,yeccpars2_53_/1}).
 -file("xref_parser.yrl", 41).
 yeccpars2_53_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   '|||'
+                     '|||'
   end | __Stack].
 
 -compile({inline,yeccpars2_54_/1}).
 -file("xref_parser.yrl", 64).
 yeccpars2_54_(__Stack0) ->
- [__3,__2,__1 | __Stack] = __Stack0,
+ [___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   { set , __2 , __1 , __3 }
+                           {set, ___2, ___1, ___3}
   end | __Stack].
 
 -compile({inline,yeccpars2_55_/1}).
 -file("xref_parser.yrl", 65).
 yeccpars2_55_(__Stack0) ->
- [__3,__2,__1 | __Stack] = __Stack0,
+ [___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   { set , __2 , __1 , __3 }
+                            {set, ___2, ___1, ___3}
   end | __Stack].
 
 -compile({inline,yeccpars2_56_/1}).
 -file("xref_parser.yrl", 68).
 yeccpars2_56_(__Stack0) ->
- [__3,__2,__1 | __Stack] = __Stack0,
+ [___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   { path , __1 , __3 }
+                            {path, ___1, ___3}
   end | __Stack].
 
 -compile({inline,yeccpars2_57_/1}).
 -file("xref_parser.yrl", 67).
 yeccpars2_57_(__Stack0) ->
- [__3,__2,__1 | __Stack] = __Stack0,
+ [___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   { restr , __2 , __1 , __3 }
+                             {restr, ___2, ___1, ___3}
   end | __Stack].
 
 -compile({inline,yeccpars2_58_/1}).
 -file("xref_parser.yrl", 69).
 yeccpars2_58_(__Stack0) ->
- [__2,__1 | __Stack] = __Stack0,
+ [___2,___1 | __Stack] = __Stack0,
  [begin
-   { type , { convert , __1 } , __2 }
+                       {type, {convert, ___1}, ___2}
   end | __Stack].
 
 -compile({inline,yeccpars2_59_/1}).
 -file("xref_parser.yrl", 62).
 yeccpars2_59_(__Stack0) ->
- [__2,__1 | __Stack] = __Stack0,
+ [___2,___1 | __Stack] = __Stack0,
  [begin
-   type ( __1 , __2 )
+                        type(___1, ___2)
   end | __Stack].
 
 -compile({inline,yeccpars2_60_/1}).
 -file("xref_parser.yrl", 66).
 yeccpars2_60_(__Stack0) ->
- [__2,__1 | __Stack] = __Stack0,
+ [___2,___1 | __Stack] = __Stack0,
  [begin
-   prefix ( __1 , __2 )
+                        prefix(___1, ___2)
   end | __Stack].
 
 -compile({inline,yeccpars2_61_/1}).
 -file("xref_parser.yrl", 70).
 yeccpars2_61_(__Stack0) ->
- [__2,__1 | __Stack] = __Stack0,
+ [___2,___1 | __Stack] = __Stack0,
  [begin
-   prefix ( __1 , __2 )
+                         prefix(___1, ___2)
   end | __Stack].
 
 -compile({inline,yeccpars2_63_/1}).
 -file("xref_parser.yrl", 96).
 yeccpars2_63_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   check_regexp_variable ( __1 )
+                     check_regexp_variable(___1)
+  end | __Stack].
+
+-compile({inline,yeccpars2_64_/1}).
+-file("xref_parser.yrl", 87).
+yeccpars2_64_(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                    ___1
   end | __Stack].
 
 -compile({inline,yeccpars2_66_/1}).
 -file("xref_parser.yrl", 88).
 yeccpars2_66_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { atom , __1 }
+                {atom, ___1}
+  end | __Stack].
+
+-compile({inline,yeccpars2_68_/1}).
+-file("xref_parser.yrl", 93).
+yeccpars2_68_(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                   ___1
+  end | __Stack].
+
+-compile({inline,yeccpars2_69_/1}).
+-file("xref_parser.yrl", 91).
+yeccpars2_69_(__Stack0) ->
+ [___1 | __Stack] = __Stack0,
+ [begin
+                   ___1
   end | __Stack].
 
 -compile({inline,yeccpars2_70_/1}).
 -file("xref_parser.yrl", 102).
 yeccpars2_70_(__Stack0) ->
  [begin
-   unknown
+                       unknown
   end | __Stack0].
 
 -compile({inline,yeccpars2_71_/1}).
 -file("xref_parser.yrl", 92).
 yeccpars2_71_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   { integer , value_of ( __1 ) }
+                    {integer, value_of(___1)}
   end | __Stack].
 
 -compile({inline,yeccpars2_72_/1}).
--file("xref_parser.yrl", 85).
+-file("xref_parser.yrl", 84).
 yeccpars2_72_(__Stack0) ->
- [__6,__5,__4,__3,__2,__1 | __Stack] = __Stack0,
+ [___6,___5,___4,___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   regexp ( func , { __1 , __3 , __5 } , __6 )
+                                                
+                     regexp(func, {___1, ___3, ___5}, ___6)
   end | __Stack].
 
 -compile({inline,yeccpars2_73_/1}).
 -file("xref_parser.yrl", 83).
 yeccpars2_73_(__Stack0) ->
- [__2,__1 | __Stack] = __Stack0,
+ [___2,___1 | __Stack] = __Stack0,
  [begin
-   regexp ( atom , __1 , __2 )
+                        regexp(atom, ___1, ___2)
   end | __Stack].
 
 -compile({inline,yeccpars2_75_/1}).
 -file("xref_parser.yrl", 56).
 yeccpars2_75_(__Stack0) ->
- [__3,__2,__1 | __Stack] = __Stack0,
+ [___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   [ __1 | __3 ]
+                                         [___1 | ___3]
   end | __Stack].
 
 -compile({inline,yeccpars2_77_/1}).
 -file("xref_parser.yrl", 34).
 yeccpars2_77_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   user
+                    user
   end | __Stack].
 
 -compile({inline,yeccpars2_78_/1}).
 -file("xref_parser.yrl", 33).
 yeccpars2_78_(__Stack0) ->
- [__1 | __Stack] = __Stack0,
+ [___1 | __Stack] = __Stack0,
  [begin
-   tmp
+                   tmp
   end | __Stack].
 
 -compile({inline,yeccpars2_79_/1}).
 -file("xref_parser.yrl", 58).
 yeccpars2_79_(__Stack0) ->
- [__3,__2,__1 | __Stack] = __Stack0,
+ [___3,___2,___1 | __Stack] = __Stack0,
  [begin
-   { assign , __2 , __1 , __3 }
+                                       {assign, ___2, ___1, ___3}
   end | __Stack].
 
 
